<?php
/**
* @package msta calendar extension
*/

namespace msta\calendar\controller;

class main {
  var $u_action;
  protected $config; /** @var \phpbb\config\config */
  protected $db; /** @var \phpbb\db\driver\driver_interface */
  protected $helper; /** @var \phpbb\controller\helper */
  protected $template; /** @var \phpbb\template\template */
  protected $user; /** @var \phpbb\user */
  protected $request; /** @var \phpbb\request\request_interface */
  protected $php_ext; /** @var string php file extension */
  protected $phpbb_root_path; /** @var string phpbb root path */
  private $table_prefix;
  private $renderer = null;
  private $importer = null;
  private $passLength = 20;

  const MAX_DAYS_DEFAULT = 365;

  public function __construct(\phpbb\config\config $config, \phpbb\db\driver\driver_interface $db, \phpbb\controller\helper $helper, \phpbb\auth\auth $auth, \phpbb\template\template $template, \phpbb\user $user, \phpbb\request\request $request, $phpbb_root_path, $phpEx) {
    $this->config = $config;
    $this->db = $db;
    $this->helper = $helper;
    $this->template = $template;
    $this->user = $user;
    $this->auth = $auth;
    $this->request = $request;
    $this->phpex = $phpEx;
    $this->phpbb_root_path = $phpbb_root_path;
    $this->ext_root_path = $phpbb_root_path.'ext/msta/calendar/';
    $this->table_prefix = substr(USERS_TABLE, 0, -5);
  }

  public function display () {
    $action = $this->request->variable('action', 'calendarmonth');
    if ($action === 'ics') return $this->generate_ics();
    if ($action === 'importics') return $this->import_ics();
    if ($action === 'filterinfo') return $this->print_filter();
    if ($action === 'calendarlist') return $this->display_list();
    if ($action === 'addpromoter' || $action === 'removepromoter') return new \Symfony\Component\HttpFoundation\Response($this->add_remove_promoter(), 200);
    //if ($action === 'addPWTable') $this->add_password_table();
    //if ($action === 'correctTimezones') $this->correct_all_timezones();
    if ($action === 'calendarmonth') return $this->display_calendar();
    return new \Symfony\Component\HttpFoundation\Response('Unknown action for calendar!', 404);
  }

  private function add_remove_promoter () {
    if (!$this->user->data['is_registered']) return 'NOT LOGGED IN!';
    if (!$this->renderer) $this->renderer = new \msta\calendar\includes\CalendarRenderer($this->db, $this->config, $this->user, $this->auth, $this->template, $this->request, $this->phpbb_root_path);
    $uid = intval($this->user->data['user_id']);
    $promoter = intval($this->request->variable('promoter', 0));
    if ($promoter <= 0) return 'NO PROMOTER GIVEN!';
    $pid = intval($this->request->variable('p', 0));
    if ($pid <= 0) return 'NO POST GIVEN!';
    $adminOrGlobalMod = $this->auth->acl_get('a_') == 1 || $this->auth->acl_get('m_') == 1;
    if (!$this->renderer->promoterCanPublish($promoter, $adminOrGlobalMod)) return 'USER NOT ALLOWED TO PUBLISH FOR THIS PROMOTER!';
    $date = $this->renderer->getDate($pid);
    if (!$date) return 'GIVEN DATE DOES NOT EXIST!';
    if ($promoter == $date['promoter']) return 'MAIN PROMOTER CAN BE CHANGED BY EDITING THE POST ONLY!';
    $action = $this->request->variable('action', 'calendarmonth');
    if ($action == 'addpromoter') {
      if ($date['co_promoters'])
        for ($i = 0; $i < count($date['co_promoters']); $i++)
          if (intval($date['co_promoters'][$i]['promoter']) == $promoter) {
            $this->db->sql_query('UPDATE '.$this->table_prefix."calendar_co_promoters SET comment = '".$this->db->sql_escape(utf8_encode_ucr($this->request->variable('comment', '', true)))."' WHERE post_id = ".$pid.' AND promoter = '.$promoter);
            return 'PROMOTER COMMENT UPDATED.';
          }
      $this->db->sql_query('INSERT INTO '.$this->table_prefix."calendar_co_promoters (post_id, promoter, comment) VALUES (".$pid.", ".$promoter.", '".$this->db->sql_escape(utf8_encode_ucr($this->request->variable('comment', '', true)))."')");
      return 'PROMOTER ADDED.';
    } else if ($action == 'removepromoter') {
      $found = false;
      if ($date['co_promoters'])
        for ($i = 0; $i < count($date['co_promoters']); $i++)
          if (intval($date['co_promoters'][$i]['promoter']) == $promoter) {
            $found = true;
            break;
          }
      if ($found) {
        $this->db->sql_query('DELETE FROM '.$this->table_prefix.'calendar_co_promoters WHERE post_id = '.$pid.' AND promoter = '.$promoter);
        return 'PROMOTER DELETED.';
      } else return 'PROMOTER WAS NO SET!';
    } else return 'UNKNOWN ACTION!';
  }

  private function import_ics () {
    $report = array();
    $settingsRow = $this->db->sql_fetchrow($this->db->sql_query('SELECT ext.*, top.topic_title FROM '.$this->table_prefix.'calendar_external AS ext INNER JOIN '.$this->table_prefix.'topics AS top ON ext.topic_id = top.topic_id WHERE ext.topic_id = '.intval($this->request->variable('id', 0))));
    if (!$settingsRow)
      return new \Symfony\Component\HttpFoundation\Response('No external calendar configured for given (topic) id!', 412);
    if ($settingsRow['trigger_pwd'] == '' || $settingsRow['trigger_pwd'] != $this->request->variable('pwd', ''))
      return new \Symfony\Component\HttpFoundation\Response('Wrong passwort to run import trigger!', 401);
    if ($this->request->file('icsfile')['tmp_name'] != '') {
      if ($this->auth->acl_get('a_') != 1)
        return new \Symfony\Component\HttpFoundation\Response('Missing permissions for file import!', 401);
      $report[] = 'Importing ICS from file...';
      $data = file_get_contents($this->request->file('icsfile')['tmp_name']);
    } else if ($settingsRow['source'] != '') {
      $report[] = 'Importing ICS from URL ['.htmlspecialchars_decode($settingsRow['source']).']...';
      $data = file_get_contents(htmlspecialchars_decode($settingsRow['source']));
    } else return new \Symfony\Component\HttpFoundation\Response("No source given!", 406);
    if (!$this->importer) $this->importer = new \msta\calendar\includes\CalendarImporter($this->db, $this->config, $this->user, $this->auth, $this->phpbb_root_path);
    $ics = $this->importer->parseICS($data);
    $report = array_merge($report, $this->importer->importEvents($ics, $settingsRow));
    return new \Symfony\Component\HttpFoundation\Response("Import done. Report:\n".join("\n", $report), 200);
  }

  private function generatePassword ($length) {
    $result = '';
    for ($i = 0; $i < $length; $i++) {
      $c = mt_rand(0, 61);
      if ($c < 10) $result .= chr($c+48);
      else if ($c < 36) $result .= chr($c-10+65);
      else $result .= chr($c-36+97);
    }
    return $result;
  }

  private function display_list () {
    $sync_user_id = '';
    $sync_password = '';
    if ($this->user->data['is_registered']) {
      $sync_user_id = $this->user->data['user_id'];
      $sync_password = $this->user->data['calendar_password'];
      if (!$sync_password || strlen($sync_password) < $this->passLength) { // user hasn't a sync-password yet. Create one.
        $sync_password = $this->generatePassword($this->passLength);
        $this->db->sql_query('UPDATE '.$this->table_prefix."users SET calendar_password = '".$sync_password."' WHERE user_id = ".$sync_user_id);
      }
    }
    $this->template->assign_vars(array(
      'SYNC_USER_ID' => $sync_user_id,
      'SYNC_PASSWORD' => $sync_password,
      'U_CALENDAR' => (($this->request->is_secure())?'https://':'http://').$this->request->server('HTTP_HOST').$this->helper->route('msta_calendar_controller'),
      'S_CALENDAR_LIST' => true,
      'CALENDAR_FILTER_PROMOTERS' => ($this->request->variable('filterpromoters', '') == '')?'':'filterpromoters="'.htmlspecialchars($this->request->variable('filterpromoters', '')).'"',
      'CALENDAR_FILTER_CATEGORIES' => ($this->request->variable('filtercategories', '') == '')?'':'filtercategories="'.htmlspecialchars($this->request->variable('filtercategories', '')).'"',
    ));
    return $this->helper->render('calendar_list.html', $this->user->lang['CALENDAR_TITLE']);
  }

  private function print_filter () {
    $filter = '{"promoters":[';
    $sql_result = $this->db->sql_query('SELECT group_id, group_name, calendar_settings FROM '.$this->table_prefix.'groups WHERE calendar_settings <> 0 ORDER BY group_name');
    while ($row = $this->db->sql_fetchrow($sql_result))
      if (($row['calendar_settings'] & 1) == 1)
        $filter .= (($filter{strlen($filter)-1} == '[')?'':', ').'{"id":'.intval($row['group_id']).', "title":'.json_encode(htmlspecialchars_decode($row['group_name'])).'}';
    $filter .= '], "categories":[';
    $sql_result = $this->db->sql_query('SELECT category_id, name, color, big, bcolor FROM '.$this->table_prefix.'calendar_category ORDER BY sort');
    while ($row = $this->db->sql_fetchrow($sql_result))
      $filter .= (($filter{strlen($filter)-1} == '[')?'':', ').'{"id":'.intval($row['category_id']).', "title":'.json_encode(htmlspecialchars_decode($row['name'])).', "color":'.json_encode(htmlspecialchars_decode($row['color'])).', "bcolor":'.json_encode(htmlspecialchars_decode($row['bcolor'])).', "big":'.(($row['big'] == 1)?'true':'false').'}';
    $filter .= '], "maxDays": ';
    if ($this->config['calendar_max_timerange']) $maxDays = intval($this->config['calendar_max_timerange']);
    else $maxDays = self::MAX_DAYS_DEFAULT;
    $filter .= $maxDays.' }';
    return new \Symfony\Component\HttpFoundation\Response($filter, 200);
  }

  private function add_password_table () {
    $this->db->sql_query('ALTER TABLE '.$this->table_prefix.'users ADD calendar_password varchar('.$this->passLength.')');
    die('altered');
  }

  private function correct_all_timezones () {
    $dates = $this->db->sql_query('SELECT * FROM '.$this->table_prefix."calendar WHERE time_from <> ''");
    while ($date = $this->db->sql_fetchrow($dates)) {
      // get
      $d_f = $date['date_from'];
      $t_f = $date['time_from'];
      $d_t = $date['date_to'];
      $t_t = $date['time_to'];
      // correct
      $dt = new \DateTime($date['date_from'].' '.$date['time_from'], $this->renderer->user_timezone);
      $dt->setTimezone($this->renderer->utc_timezone);
      $d_f = $dt->format('Y-m-d');
      $t_f = $dt->format('H:i');
      if ($date['repeat_dm'] == 0) {
        $dt = new \DateTime($date['date_to'].' '.$date['time_to'], $this->renderer->user_timezone);
        $dt->setTimezone($this->renderer->utc_timezone);
        $d_t = $dt->format('Y-m-d');
        $t_t = $dt->format('H:i');
      } else { // for repeating dates, the timeshift belongs to the beginning date
        $dt = new \DateTime($date['date_from'].' '.$date['time_to'], $this->renderer->user_timezone);
        $dt->setTimezone($this->renderer->utc_timezone);
        $t_t = $dt->format('H:i');
      }
      // apply
      $this->db->sql_query('UPDATE '.$this->table_prefix."calendar SET date_from='".$d_f."', time_from='".$t_f."', date_to='".$d_t."', time_to='".$t_t."' WHERE post_id = ".$date['post_id']);
    }
    die("All Dates updated! Do not repeat this!");
  }

  private function paramToDateTime ($par) {
    switch (substr($par, 0, 1)) {
      case 'D':
        $d = intval(substr($par, 1));
        $dt = new \DateTime();
        $dt->setTimezone($this->renderer->utc_timezone);
        if ($d >= 0) $dt->add(new \DateInterval('P'.$d.'D'));
        else $dt->sub(new \DateInterval('P'.(-$d).'D'));
        return $dt;
      case 'U':
        $dt = \DateTime::createFromFormat('U', substr($par, 1), $this->renderer->utc_timezone);
        if ($dt === false) {
          $dt = new \DateTime();
          $dt->setTimezone($this->renderer->utc_timezone);
        }
        return $dt;
      default:
        $dt = \DateTime::createFromFormat('Y-m-d', $par, $this->renderer->user_timezone);
        if ($dt === false) $dt = new \DateTime();
        $dt->setTimezone($this->renderer->utc_timezone);
        return $dt;
    }
  }

  private function generate_ics () {
    if (!$this->renderer) $this->renderer = new \msta\calendar\includes\CalendarRenderer($this->db, $this->config, $this->user, $this->auth, $this->template, $this->request, $this->phpbb_root_path);
    $this->user->add_lang_ext('msta/calendar', 'calendar');
    // user authentification
    $uid = 0;
    $vUser = $this->request->variable('user', 'public');
    if ($vUser != 'public') {
      if (($vUser == 'current') || (intval($vUser) == intval($this->user->data['user_id']))) {
        if ($this->user->data['is_registered']) $uid = $this->user->data['user_id'];
      } else {
        $uid = intval($vUser);
        if (!$this->request->is_secure()) {
          $uid = 0;
          return new \Symfony\Component\HttpFoundation\Response('Only secure connection is allowed for user based calender synchronisazion!', 400);
        }
        $sql_row = $this->db->sql_fetchrow($this->db->sql_query('SELECT calendar_password FROM '.$this->table_prefix.'users WHERE (user_type = '.USER_NORMAL.' OR user_type = '.USER_FOUNDER.') AND user_id = '.$uid));
        if ((!$sql_row) || (strlen($sql_row['calendar_password']) < $this->passLength) || ($sql_row['calendar_password'] != $this->request->variable('password', ''))) {
          $uid = 0;
          return new \Symfony\Component\HttpFoundation\Response('User or passwort wrong. Use the user-id and the special calendar-passwort for login!', 401);
        }
      }
    }
    // parse request filter variables
    $promoters = array();
    $categories = array();
    $promBl = false;
    $catBl = false;
    $rq = explode(',', $this->request->variable('promoters', ''));
    if (count($rq) == 1) $rq = explode('|', $this->request->variable('promoters', '')); // fallback to old syntax, which was changed, because not all programs accept a | in URLs
    for ($i = 0; $i < count($rq); $i++)
      if ($rq[$i] == '~' && $i == 0) $promBl = true;
      else if (intval($rq[$i]) > 0) $promoters[] = intval($rq[$i]);
    $rq = explode(',', $this->request->variable('categories', ''));
    if (count($rq) == 1) $rq = explode('|', $this->request->variable('categories', '')); // fallback to old syntax, which was changed, because not all programs accept a | in URLs
    for ($i = 0; $i < count($rq); $i++)
      if ($rq[$i] == '~' && $i == 0) $catBl = true;
      else if (intval($rq[$i]) > 0) $categories[] = intval($rq[$i]);
    // prepare time range
    if ($this->config['calendar_max_timerange']) $maxDays = intval($this->config['calendar_max_timerange']);
    else $maxDays = self::MAX_DAYS_DEFAULT;
    $from = $this->paramToDateTime($this->request->variable('from', 'D-'.ceil($maxDays/2.0)));
    $to = $this->paramToDateTime($this->request->variable('to', ''));
    $dayDiff = intval($from->diff($to)->format('%r%a'));
    if ($dayDiff < 0) $from = clone($to);
    else if ($dayDiff > $maxDays) $from = $this->paramToDateTime('D-'.ceil($maxDays/2.0));
    if ($this->request->variable('to', '') == '') $to = null; // no time limit for future events!
    // get dates
    $dates = $this->renderer->getDates($uid, $from, $to, $promoters, $categories, (($uid == 0)?8:0)|(($promBl)?1:0)|(($catBl)?2:0));
    // generate response
    $ics = $this->renderer->datesToICS($dates, $uid, $promoters, $promBl, $this->request->server('SERVER_NAME'), $this->request->variable('descriptionextend', ''));
    return new \Symfony\Component\HttpFoundation\Response($ics, 200);
  }

  private function display_calendar () {
    if (!$this->renderer) $this->renderer = new \msta\calendar\includes\CalendarRenderer($this->db, $this->config, $this->user, $this->auth, $this->template, $this->request, $this->phpbb_root_path);
    // compose time interval
    $now = new \DateTime('now', $this->renderer->user_timezone);
    $month = intval($this->request->variable('new_month', $this->request->variable('month', $now->format('m'))));
    $year = intval($this->request->variable('new_year', $this->request->variable('year', $now->format('Y'))));
    $from = new \DateTime(str_pad($year, 4, '0', STR_PAD_LEFT).'-'.str_pad($month, 2, '0', STR_PAD_LEFT).'-00', $this->renderer->user_timezone);
    $from->setTimezone($this->renderer->utc_timezone);
    $prevMonth = clone $from;
    $prevMonth->sub(new \DateInterval('P1D'));
    $nextMonth = clone $from;
    $nextMonth->add(new \DateInterval('P33D'));
    $fromUserTZ = clone $from;
    $fromUserTZ->setTimezone($this->renderer->user_timezone);
    $from->sub(new \DateInterval('P'.((intval($fromUserTZ->format('N')) + 7 - intval($this->config['calendar_week_start'])) % 7).'D')); // go back to week start
    $to = clone $from;
    $to->add(new \DateInterval('P42D'));
    // create dates and days-list
    $dates = $this->renderer->getDates($this->user->data['user_id'], $from, $to, array(), array(), (($this->config['calendar_birthday_on_calendar'] == 1)?4:0));
    $dayList = $this->renderer->getDayList($from, 42);
    $this->renderer->mapDatesToDays($dates, $dayList);
    // template vars
    $this->template->assign_vars(array(
      'U_CALENDAR'		=> (($this->request->is_secure())?'https://':'http://').$this->request->server('HTTP_HOST').$this->helper->route('msta_calendar_controller'),
      'S_IN_CALENDAR'	=> true, // TODO: This may be unnecessary
      'S_CALENDAR'		=> true,
      'CALENDAR'		=> false,
      'PREVIOUS'		=> append_sid('?month='.$prevMonth->format('m').'&amp;year='.$prevMonth->format('Y')),
      'NEXT'			=> append_sid('?month='.$nextMonth->format('m').'&amp;year='.$nextMonth->format('Y')),
      'U_ACTION'		=> $this->u_action,
      'WEEK'			=> 0, // TODO: what's this?
      'EVENT_LEGEND'	=> $this->config['calendar_legend_display'],
    ));
    // render
    $this->user->add_lang_ext('msta/calendar', 'calendar');
    $this->renderer->renderDayNameList();
    $this->renderer->renderMonthNameList($month);
    $this->renderer->renderYearList($year);
    $this->renderer->renderDays($dayList, $month);
    return $this->helper->render('calendar_body.html', $this->user->lang['CALENDAR_TITLE']);
  }
}
