<?php
/**
* @package msta calendar extension
*/

namespace msta\calendar\event;

/**
* @ignore
*/
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class main_listener implements EventSubscriberInterface {
  static public function getSubscribedEvents () {
    return array(
      'core.user_setup'								=> 'load_language_on_setup',
      'core.posting_modify_template_vars'   		=> 'calendar',
      'core.page_header'     						=> 'calendar_on_header',
      'core.viewonline_overwrite_location'			=> 'viewonline_page',
      'core.delete_posts_after'						=> 'delete_posts',
      'core.submit_post_end'						=> 'send_data_to_table',
      'core.viewtopic_assign_template_vars_before'	=> 'participate_from_form',
      'core.viewtopic_modify_post_row'				=> 'display_participants_list',
      'msta.emailontopic.receivers_prepare_list'	=> 'receivers_prepare_list',
      'msta.emailontopic.receivers_receive_list'	=> 'receivers_receive_list',
      'msta.emailontopic.send_bridges'				=> 'calendar_in_email',
      'msta.emailontopic.mail_access_trigger'		=> 'participate_from_mail',
    );
  }

  protected $config;
  protected $db;
  protected $helper;
  protected $template;
  protected $user;
  protected $auth;
  protected $request;
  protected $phpbb_root_path;
  protected $phpbb_dispatcher;	
  private $table_prefix;
  private $utc_timezone = null;
  private $user_timezone = null;
  private $renderer = null;

  public function __construct (\phpbb\config\config $config, \phpbb\db\driver\driver_interface $db, \phpbb\controller\helper $helper, \phpbb\auth\auth $auth, \phpbb\template\template $template, \phpbb\user $user, \phpbb\request\request $request, \phpbb\event\dispatcher_interface $phpbb_dispatcher, $phpbb_root_path) {
    $this->config = $config;
    $this->db = $db;
    $this->helper = $helper;
    $this->template = $template;
    $this->user = $user;
    $this->auth = $auth;
    $this->request = $request;
    $this->phpbb_root_path = $phpbb_root_path;
    $this->ext_root_path = $phpbb_root_path . 'ext/msta/calendar/';
    $this->phpbb_dispatcher = $phpbb_dispatcher;
    $this->table_prefix = substr(USERS_TABLE, 0, -5);
    $this->utc_timezone = new \DateTimeZone('UTC');
    if ($this->user->data['user_timezone']) $this->user_timezone = new \DateTimeZone($this->user->data['user_timezone']);
    else $this->user_timezone = new \DateTimeZone($this->config['board_timezone']);
  }

  public function receivers_prepare_list ($event) {
    if ($event['postId'] == 0) return;
    $sqlRes = $this->db->sql_query('SELECT part.user_id, u.username, g.group_id, g.group_name FROM '.$this->table_prefix."calendar_participants AS part LEFT JOIN ".$this->table_prefix."groups AS g ON part.user_id < 0 AND part.user_id = - g.group_id LEFT JOIN ".$this->table_prefix."users AS u ON part.user_id > 0 AND part.user_id = u.user_id WHERE part.post_id = ".intval($event['postId'])." AND part.participate = 4");
    $list = $event['list'];
    $mod = false;
    while ($row = $this->db->sql_fetchrow($sqlRes)) {
      if (array_key_exists('U'.intval($row['user_id']), $list)) $list['U'.intval($row['user_id'])]['calInvite'] = true;
      else $list['U'.intval($row['user_id'])] = [
        'name' => ($row['user_id'] < 0)?$row['group_name']:$row['username'],
        'isNew' => true,
        'isGroup' => $row['user_id'] < 0,
        'action' => 0,
        'calInvite' => true,
        'uid' => ($row['user_id'] > 0)?intval($row['user_id']):0,
        'gid' => ($row['user_id'] < 0)?-intval($row['user_id']):0,
      ];
      $mod = true;
    }
    if ($mod) $event['list'] = $list;
  }

  public function receivers_receive_list ($event) {
    foreach ($event['users'] as $user)
      $this->modify_participants_list($user['uid'], $event['postId'], 0, 0, ($user['calInvite'])?4:-4, '');
    foreach ($event['groups'] as $group)
      $this->modify_participants_list(-$group['gid'], $event['postId'], 0, 0, ($group['calInvite'])?4:-4, '');
  }

  public function viewonline_page ($event) {
    switch ($event['on_page'][1]) {
      case 'app':
        if (strrpos($event['row']['session_page'], '/calendar')) {
          $event['location'] = $this->user->lang('VIEWING_CALENDAR');
          $event['location_url'] = $this->helper->route('msta_calendar_controller');
        }
        break;
    }
  }

  public function delete_posts ($event) {
    $calUtil = new \msta\calendar\includes\CalendarImporter($this->db, $this->config, $this->user, $this->auth, $this->phpbb_root_path);
    for ($i = 0; $i < count($event['post_ids']); $i++)
      $calUtil->deleteEvent(intval($event['post_ids'][$i]));
  }

  public function load_language_on_setup ($event) {
    $lang_set_ext = $event['lang_set_ext'];
    $lang_set_ext[] = array(
      'ext_name' => 'msta/calendar',
      'lang_set' => 'calendar',
    );
    $event['lang_set_ext'] = $lang_set_ext;
  }

  public function calendar_in_email ($event) {
    if (intval($event['postId']) == 0) return;
    if (!$this->renderer) $this->renderer = new \msta\calendar\includes\CalendarRenderer($this->db, $this->config, $this->user, $this->auth, $this->template, $this->request, $this->phpbb_root_path);
    if ($this->request->variable('d', 0) == 0) $currentDate = new \DateTime();
    else $currentDate = \DateTime::createFromFormat('U', $this->request->variable('d', 0));
    $date = $this->renderer->getDate(intval($event['postId']));
    if (!$date) return;
    $vars = $event['vars'];
    $vars['templates_before_message'][] = '@msta_calendar/date_details.html';
    $vars['postrow'] = array();
    $vars['postrow']['CALENDAR'] = $this->renderer->renderDate($date, $currentDate, false, false);
    $vars['postrow']['CALENDAR_CO_PROMOTERS'] = $vars['postrow']['CALENDAR']['CO_PROMOTERS']; // because nested lists don't work...
    $vars['postrow']['CALENDAR']['CO_PROMOTERS'] = $vars['postrow']['CALENDAR_CO_PROMOTERS'] !== null;
    $vars['postrow']['CALENDAR_PARTICIPANTS'] = $vars['postrow']['CALENDAR']['PARTICIPANTS']; // because nested lists don't work...
    $vars['postrow']['CALENDAR']['PARTICIPANTS'] = $vars['postrow']['CALENDAR_PARTICIPANTS'] !== null;
    $event['vars'] = $vars;
    $event['mail_access_triggers'] = array_merge($event['mail_access_triggers'], array(
      'CALENDAR_BUT_ASSURE' => $vars['postrow']['CALENDAR']['EVENT_DATE'],
      'CALENDAR_BUT_MAYBE' => $vars['postrow']['CALENDAR']['EVENT_DATE'],
      'CALENDAR_BUT_REJECT' => $vars['postrow']['CALENDAR']['EVENT_DATE'],
    ));
  }

  public function display_participants_list ($event) {
    if (!$this->renderer) $this->renderer = new \msta\calendar\includes\CalendarRenderer($this->db, $this->config, $this->user, $this->auth, $this->template, $this->request, $this->phpbb_root_path);
    if ($this->request->variable('d', 0) == 0) $currentDate = new \DateTime();
    else $currentDate = \DateTime::createFromFormat('U', $this->request->variable('d', 0));
    $date = $this->renderer->getDate(intval($event['row']['post_id']));
    if (!$date) return;
    $cp = $event['cp_row'];
    $cp['row']['CALENDAR'] = $this->renderer->renderDate($date, $currentDate, true, true);
    $cp['row']['CALENDAR_CO_PROMOTERS'] = $cp['row']['CALENDAR']['CO_PROMOTERS']; // because nested lists don't work...
    $cp['row']['CALENDAR']['CO_PROMOTERS'] = $cp['row']['CALENDAR_CO_PROMOTERS'] !== null;
    $cp['row']['CALENDAR_PARTICIPANTS'] = $cp['row']['CALENDAR']['PARTICIPANTS']; // because nested lists don't work...
    $cp['row']['CALENDAR']['PARTICIPANTS'] = $cp['row']['CALENDAR_PARTICIPANTS'] !== null;
    $event['cp_row'] = $cp;
  }

  public function calendar_on_header ($event) {
    if (strpos($this->request->server('REQUEST_URI'), $this->helper->route('msta_calendar_controller')) === 0) return;
    // template vars
    $this->template->assign_vars(array(
      'U_CALENDAR' 				=> (($this->request->is_secure())?'https://':'http://').$this->request->server('HTTP_HOST').$this->helper->route('msta_calendar_controller'),
      'S_WEEK_OR_NEXT'			=> $this->config['calendar_week_or_next'],
      'S_WEEK_ON_INDEX'			=> $this->config['calendar_week_on_index'],
      'S_BIRTHDAY_ON_CALENDAR'	=> $this->config['calendar_birthday_on_calendar'],
      'S_CALENDAR_FOR_GUESTS'	=> $this->config['calendar_for_guests'],
      'CALENDAR'				=> ($this->config['calendar_on_index_off'])?false:true,
    ));	
    if ($this->config['calendar_on_index_off']) return; // TODO: Original code does something with cookies here...
    // compose time interval
    if (!$this->renderer)
      $this->renderer = new \msta\calendar\includes\CalendarRenderer($this->db, $this->config, $this->user, $this->auth, $this->template, $this->request, $this->phpbb_root_path);
    $from = new \DateTime('now', $this->renderer->user_timezone);
    $from = new \DateTime($from->format('Y-m-d'), $this->renderer->user_timezone); // go back to day begin
    $from->setTimezone($this->renderer->utc_timezone);
    $to = clone $from;
    $to->add(new \DateInterval('P7D'));
    // create dates and days-list
    $dates = $this->renderer->getDates($this->user->data['user_id'], $from, $to, array(), array(), (($this->config['calendar_birthday_on_calendar'] == 1)?4:0));
    $dayList = $this->renderer->getDayList($from, 7);
    $this->renderer->mapDatesToDays($dates, $dayList);
    // render
    $this->user->add_lang_ext('msta/calendar', 'calendar');
    $this->renderer->renderDays($dayList, -1);
  }

  private function modify_participants_list ($user_id, $post_id, $event_date, $nr, $part, $comment) {
    if ($user_id <= 0 && abs($part) != 4) return false;
    $row = $this->db->sql_fetchrow($this->db->sql_query('SELECT participants FROM '.$this->table_prefix."calendar WHERE post_id = ".intval($post_id)));
    if (!$row || $row['participants'] == 0) return false; // post has no date or date has no participants!
    if (abs($part) == 4) { // ensure that user is allowed to manage invitations for this event
      $row = $this->db->sql_fetchrow($this->db->sql_query('SELECT poster_id, forum_id FROM '.$this->table_prefix."posts WHERE post_id = ".intval($post_id)));
      if (!$row) return false; // post does not exist
      if (!($this->user->data['is_registered'] && ($this->auth->acl_get('m_edit', $row['forum_id']) || ($this->auth->acl_get('f_edit', $row['forum_id']) && $this->user->data['user_id'] == $row['poster_id'])))) return false; // user has no permission to edit post
    } else { // ensure, that user is allowed to register for this event
      if ($row['participants'] == 2) {
        $row = $this->db->sql_fetchrow($this->db->sql_query('SELECT post_id FROM ' . $this->table_prefix . "calendar_participants WHERE post_id = ".intval($post_id)." AND user_id = ".intval($user_id)." AND participate = 4"));
        if (!$row) return false; // this is a closed event and the user is not invited
      }
    }
    $sql_ary = array(
      'POST_ID'		=> intval($post_id),
      'EVENT_DATE'	=> ($event_date == 0)?'':date("Y-m-d", $event_date),
      'USER_ID'		=> intval($user_id),
      'NUMBER'		=> intval($nr),
      'PARTICIPATE'	=> intval($part),
      'COMMENTS'	=> utf8_encode_ucr($comment),
      'DATE'		=> date("Y-m-d-H-i"),
    );
    $triggerResult = $this->phpbb_dispatcher->trigger_event('msta.calendar.modify_participants_list', $sql_ary);
    if ($triggerResult && gettype($triggerResult['COMMENTS']) == 'string') $sql_ary['COMMENTS'] = $triggerResult['COMMENTS'];
    if ($sql_ary['EVENT_DATE'] == '' && abs($sql_ary['PARTICIPATE'] <= 3)) return false; // cannot participate without date!
    $row = $this->db->sql_fetchrow($this->db->sql_query('SELECT * FROM ' . $this->table_prefix . "calendar_participants WHERE post_id = ".$sql_ary['POST_ID']." AND event_date = '".$sql_ary['EVENT_DATE']."' AND user_id = ".$sql_ary['USER_ID']));
    if ($part < 0) { // delete invite
      if ($part == -4) {
        $this->db->sql_query('DELETE FROM '.$this->table_prefix."calendar_participants WHERE post_id = ".$sql_ary['POST_ID']." AND event_date = '".$sql_ary['EVENT_DATE']."' AND user_id = ".$sql_ary['USER_ID']." AND participate = 4");
        $notify = 'calendar_invitation_delete';
      }
    } else { // set participation
      if ($row) {
        $this->db->sql_query('UPDATE '.$this->table_prefix.'calendar_participants SET '.$this->db->sql_build_array('UPDATE', $sql_ary)." WHERE post_id = ".$sql_ary['POST_ID']." AND event_date = '".$sql_ary['EVENT_DATE']."' AND user_id = ".$sql_ary['USER_ID']);
        $notify = 'calendar_participants_change';
      } else {
        $this->db->sql_query('INSERT INTO '.$this->table_prefix.'calendar_participants '.$this->db->sql_build_array('INSERT', $sql_ary));	
        $notify = 'calendar_participants';
      }
    }
    return true;
    //if ($this->config['calendar_notify_participating'] == true) calendar_messenger($notify, $user_id, $sql_ary['POST_ID'], 'd=' . $event_date . '&');
  }

  public function participate_from_form () {
    $this->modify_participants_list(
      intval($this->user->data['user_id']),
      intval($this->request->variable('participants_id', 0)),
      $this->request->variable('d', '', true),
      intval($this->request->variable('group', 1)),
      intval($this->request->variable('part', -1)),
      utf8_normalize_nfc($this->request->variable('comments', '', true))
    );
  }

  public function participate_from_mail ($event) {
    if ($event['trigger_name'] == 'CALENDAR_BUT_ASSURE') $part = 2;
    else if ($event['trigger_name'] == 'CALENDAR_BUT_MAYBE') $part = 1;
    else if ($event['trigger_name'] == 'CALENDAR_BUT_REJECT') $part = 0;
    else {
      $event['result'] = 'Participation could not be stored. Invalid request.';
      return;
    }
    $success = $this->modify_participants_list(
      intval($event['user_id']),
      intval($event['post_id']),
      $event['trigger_value'],
      1,
      $part,
      ''
    );
    if ($success) $event['result'] = 'Participation stored.';
    else $event['result'] = 'Participation could not be stored. You are maybe not invited to this event.';
  }

  /*
   dateFromForm
   loads the date values from form and transforms to gmt-timezone, if loaded from form. The date can exist before and if so, the timezone will be kept.
  */
  private function dateFromForm (&$date, $toTable, $calUtil) {
    if (($this->request->variable('calendar_post_id', 'not set') != 'not set') || $toTable) {
      foreach ($date as $k => $v)
        if ($k != 'post_id')
          $date[$k] = $this->request->variable('calendar_'.$k, $date[$k], true);
      // extra treatment for checkbox values
      if ($this->request->variable('calendar_public', -1) != -1) $date['public'] = 1;
      else $date['public'] = 0;
      $date['repeat_mode'] = ($this->request->variable('calendar_no_repeat', -1) == -1)?0:$this->request->variable('calendar_repeat_mode', 0);
      switch ($date['repeat_mode']) {
        case 0:
          $date['repeat_nr'] = 0;
          break;
        case 1:
          $date['repeat_nr'] = $this->request->variable('calendar_repeat_month_number', $date['repeat_nr']);
          break;
        case 2:
          $date['repeat_nr'] = $this->request->variable('calendar_repeat_day_number', $date['repeat_nr']);
          break;
        case 3:
          $date['repeat_nr'] = $this->request->variable('calendar_repeat_weak_month', $date['repeat_nr']);
          break;
      }
      if ($this->request->variable('calendar_canceled', -1) != -1) $date['canceled'] = 1;
      else $date['canceled'] = 0;
      $date['participants'] = intval($this->request->variable('calendar_participants', 0));
      if ($this->request->variable('calendar_complete_day', -1) != -1) {
        $date['time_from'] = '';
        $date['time_to'] = '';
      }
    }
    // check event data
    $calUtil->checkEvent($date);
    if ($date['time_from'] != '') { // corrent timezone if (and only if) the time was set through the form
      if ($this->request->variable('calendar_time_from', 'X', true) != 'X') {
        $dt = new \DateTime($date['date_from'].' '.$date['time_from'], $this->user_timezone);
        $dt->setTimezone($this->utc_timezone);
        $date['date_from'] = $dt->format('Y-m-d');
        $date['time_from'] = $dt->format('H:i');
      }
      if ($this->request->variable('calendar_time_to', 'X', true) != 'X') {
        if ($date['repeat_mode'] == 0) {
          $dt = new \DateTime($date['date_to'].' '.$date['time_to'], $this->user_timezone);
          $dt->setTimezone($this->utc_timezone);
          $date['date_to'] = $dt->format('Y-m-d');
          $date['time_to'] = $dt->format('H:i');
        } else { // for repeating dates, the timeshift belongs to the beginning date
          $dt = new \DateTime($date['date_from'].' '.$date['time_to'], $this->user_timezone);
          $dt->setTimezone($this->utc_timezone);
          $date['time_to'] = $dt->format('H:i');
        }
      }
    }
  }

  public function calendar ($event) { // hands over the calender information to the website for existing dates or after preview submits
    if ($this->request->variable('mode', '') == 'quote') return;
    // hand over page data
    $event['page_data']['SUBJECT'] = $this->request->variable('subject', $event['page_data']['SUBJECT'], true);
    $event['page_data']['MESSAGE'] = $this->request->variable('message', $event['page_data']['MESSAGE'], true);
    // exit, if this is not the first post and only first post dates are allowed
    $quote = (substr($event['post_data']['post_text'], 1, 5) == 'quote');
    $first_post_id = (isset($event['post_data']['topic_first_post_id'])) ? $event['post_data']['topic_first_post_id'] : 0;
    $first_post = ($first_post_id == 0) || ($first_post_id == $event['post_data']['post_id'] && !$quote);
    if ($this->config['calendar_only_first_post'] && !$first_post) return;
    // check, if this forum allows calendar entries
    $result = $this->db->sql_query('SELECT * FROM '.$this->table_prefix.'calendar_forums WHERE forum_id = '.intval($event['forum_id']));
    $forum_row = $this->db->sql_fetchrow($result);
    $this->db->sql_freeresult($result);
    if ($forum_row && $forum_row['allowed'] == 1) {
      $calUtil = new \msta\calendar\includes\CalendarBase($this->db, $this->config, $this->user);
      $this->template->assign_vars(array('CALENDAR_ALLOWED' => true));
      // create an empty date
      $date = array();
      $calUtil->checkEvent($date);
      // load existing event from database
      $dateExists = false;
      if ($event['post_data']['post_id']) {
        $result = $this->db->sql_query('SELECT * FROM '.$this->table_prefix.'calendar WHERE post_id = '.intval($event['post_data']['post_id']));
        $date_row = $this->db->sql_fetchrow($result);
        if ($date_row) {
          foreach ($date as $k => $v)
            if (array_key_exists($k, $date_row))
              $date[$k] = $date_row[$k];
          $dateExists = true;
        }
      }
      // load information from form
      $this->dateFromForm($date, false, $calUtil);
      if ($this->request->variable('calendar_post_id', -1) != -1)
        $dateExists = $this->request->variable('calendar_date_exists', -1) != -1;
      // convert to user timezone or set to empty for new dates
      if ($dateExists) {
        if ($date['time_from'] != '') {
          $dt = new \DateTime($date['date_from'].' '.$date['time_from'], $this->utc_timezone);
          $dt->setTimezone($this->user_timezone);
          $date['date_from'] = $dt->format('Y-m-d');
          $date['time_from'] = $dt->format('H:i');
          if ($date['repeat_mode'] == 0) {
            $dt = new \DateTime($date['date_to'].' '.$date['time_to'], $this->utc_timezone);
            $dt->setTimezone($this->user_timezone);
            $date['date_to'] = $dt->format('Y-m-d');
            $date['time_to'] = $dt->format('H:i');
          } else { // for repeating dates, the timeshift belongs to the beginning date
            $dt = new \DateTime($date['date_from'].' '.$date['time_to'], $this->utc_timezone);
            $dt->setTimezone($this->user_timezone);
            $date['time_to'] = $dt->format('H:i');
          }
        }
      } else {
        $date['date_from'] = '';
        $date['time_from'] = '';
        $date['date_to'] = '';
        $date['time_to'] = '';
      }
      // set template vars
      foreach ($date as $k => $v)
        $this->template->assign_vars(array('calendar_'.$k => $v));
      $this->template->assign_vars(array(
        'calendar_anniversary' => false,
        'calendar_public_checked' => (($date['public'] == 0)?'':'checked'),
        'calendar_date_exists_checked' => (($dateExists)?'checked':''),
      ));
      // generate lists of availlable calendars/event-types and promoters
      $adminOrGlobalMod = $this->auth->acl_get('a_') == 1 || $this->auth->acl_get('m_') == 1; // check, if the current user is admin or global moderator
      $selectedPromoter = intval($date['promoter']);
      if ($selectedPromoter == 0) { // select a promoter, which has explicit write permission to this forum as default
        $promoters = $calUtil->getPromoterIDs(true);
        for ($i = 0; $i < count($promoters); $i++)
          if ($this->auth->acl_group_raw_data($promoters[$i], false, intval($event['forum_id']))[$promoters[$i]][intval($event['forum_id'])]['f_post'] == 1) {
            $selectedPromoter = $promoters[$i];
            break;
          }
      }
      $this->template->assign_vars(array(
        'calendar_categories' => $calUtil->categoriesAsOptions(intval($date['category'])),
        'calendar_promoter_options' => $calUtil->promotersAsOptions(!$adminOrGlobalMod, !$adminOrGlobalMod, $selectedPromoter),
      ));
    }
  }

  public function send_data_to_table ($event) {
    if ($this->request->variable('mode', '') == 'quote') return;
    if (!$event['data']['post_id'] || intval($event['data']['post_id']) == 0) return;
    $notify = '';
    $calUtil = new \msta\calendar\includes\CalendarImporter($this->db, $this->config, $this->user, $this->auth, $this->phpbb_root_path);
    if (intval($this->request->variable('calendar_date_exists', -1)) != -1) { // update or add
      // create empty date
      $date = array();
      $calUtil->checkEvent($date);
      // load date properties from database
      $result = $this->db->sql_query('SELECT * FROM '.$this->table_prefix.'calendar WHERE post_id = '.intval($event['data']['post_id']));
      $date_row = $this->db->sql_fetchrow($result);
      $existingDate = null;
      if ($date_row) {
        $existingDate = array();
        foreach ($date as $k => $v)
          if (array_key_exists($k, $date_row)) {
            $date[$k] = $date_row[$k];
            $existingDate[$k] = $date_row[$k];
          }
      }
      $newDate = $date['post_id'] == 0;
      // load information from form
      $this->dateFromForm($date, true, $calUtil);
      if ($date['date_from'] == '' || $date['category'] == 0) return;
      $adminOrGlobalMod = $this->auth->acl_get('a_') == 1 || $this->auth->acl_get('m_') == 1; // check, if the current user is admin or global moderator
      if (!$calUtil->promoterIsValid($date['promoter'], !$adminOrGlobalMod)) $date['promoter'] = 0;
      if (!$adminOrGlobalMod && !$calUtil->promoterCanPublish($date['promoter'])) { // user is not allowed to publish events
        if ($existingDate && $existingDate['public'] == 1) { // if this is an existing public date, maybe published by an Administrator, the user cannot modify some important fields
          $date['public'] = $existingDate['public'];
          $date['promoter'] = $existingDate['promoter'];
          $date['event_name'] = $existingDate['event_name'];
          $date['category'] = $existingDate['category'];
        } else { // if the date wasn't published yet, dont allow to publish it now!
          $date['public'] = 0;
        }
      }
      // apply
      $date['post_id'] = $event['data']['post_id'];
      $calUtil->insertOrUpdateEvent($date, $newDate);
    } else { // delete and store del entry
      if ($calUtil->deleteEvent($event['data']['post_id'], true)) $notify = 'calendar_notify_delete';
      if ($notify != '' && $this->config['calendar_notify'] == true)
        calendar_messenger($notify, $event['page_data']['poster_id'], $event['data']['post_id'], '');
    }
  }
}

function calendar_messenger($mode, $author_id, $post_id, $pd)
{
/*	global $user, $db, $phpbb_root_path, $phpEx, $config;
	include_once($phpbb_root_path . 'includes/functions_messenger.' . $phpEx);

	$sql = 'SELECT poster_id, forum_id, post_subject
			FROM ' . POSTS_TABLE . '
			WHERE post_id = ' . $post_id;
	$post_result = $db->sql_query($sql);
	$post_row = $db->sql_fetchrow($post_result);

	$sql = 'SELECT username
		FROM ' . USERS_TABLE . '
			WHERE user_id = ' . $author_id;
	$user_result = $db->sql_query($sql);
	$user_row = $db->sql_fetchrow($user_result);
	$author = $user_row['username'];
	$part_user = array();
	$poster = false;

	if(($mode == 'calendar_participants' || $mode == 'calendar_participants_change') && $config['calendar_notify_participating'] == true && $config['calendar_notify_participating_adress'] != 1)
	{
		if($config['calendar_notify_participating_adress'] != 2)
		{
			$sql = 'SELECT user_id
					FROM ' . CALENDAR_PARTICIPANTS_TABLE . '
					WHERE post_id = ' . $post_id;
			$part_result = $db->sql_query($sql);
			while($part_row = $db->sql_fetchrow($part_result))
			{
				$part_user[] = $part_row['user_id'];
				if($part_row['user_id'] == $post_row['poster_id'])
				{
					$poster = true;
				}
			}
		}

		
		if($poster == false)
		{
			$part_user[] = $post_row['poster_id'];
		}

		for($i = 0; $i < count($part_user); $i++)
		{

			$sql = 'SELECT username, user_email, user_lang, user_inactive_reason
					FROM ' . USERS_TABLE . '
					WHERE user_id = ' . $part_user[$i];
		
			$user_result = $db->sql_query($sql);
			$user_row = $db->sql_fetchrow($user_result);
			
			calendar_messenger_send($mode, $user_row['user_lang'], $user_row['user_email'], $config['sitename'], $author, $user_row['username'], $user_row['user_inactive_reason'], $post_row['post_subject'], $post_row['forum_id'], $post_id, $part_user[$i], $author_id, $pd);
		}
	}
	else
	{
		$sql = 'SELECT user_id, username, user_email, user_lang, user_inactive_reason
				FROM ' . USERS_TABLE;
		
		$user_result = $db->sql_query($sql);
		while($user_row = $db->sql_fetchrow($user_result))
		{
			calendar_messenger_send($mode, $user_row['user_lang'], $user_row['user_email'], $config['sitename'], $author, $user_row['username'], $user_row['user_inactive_reason'], $post_row['post_subject'], $post_row['forum_id'], $post_id, $user_row['user_id'] ,$author_id, $pd);
		}
	}*/
}

function calendar_messenger_send($mode, $lang, $email, $sitename, $author, $username, $inactive, $subject, $forum_id, $post_id, $user_id, $author_id, $pd)
{
	global $phpbb_root_path, $auth, $db;
	$url = generate_board_url();

	$sql = 'SELECT ban_userid
			FROM ' . BANLIST_TABLE . '
			WHERE ban_userid = ' . $user_id;
	$ban_result = $db->sql_query($sql);
	$ban_row = $db->sql_fetchrow($ban_result);
	if(!$ban_row)
	{
		if($username && $email && $lang && !$inactive && $user_id != $author_id)
		{
			$auth_array = $auth->acl_raw_data($user_id, 'f_read', $forum_id);
			if(isset($auth_array[$user_id][$forum_id]['f_read']) && $auth_array[$user_id][$forum_id]['f_read'])
			{
				$mail_template_path = $phpbb_root_path . 'ext/msta/calendar/language/' . $lang . '/email/';

				$messenger = new \messenger();
				$messenger->template($mode, $lang , $mail_template_path);
				$messenger->to($email, $sitename);
				$messenger->assign_vars(array(
					'AUTHOR'   			=> $author,
					'USERNAME'   		=> $username,
					'EVENT_NAME' 		=> $subject,
					'U_CALENDAR_EVENT' 	=> $url . '/viewtopic.php?' . $pd . 'p=' . $post_id,
				));
				$messenger->send();
			}
		}
	}
}

function upcoming_events($upcoming)
{
	global $db, $user, $auth, $phpbb_root_path, $config, $wday, $c_event, $c_c, $c_from, $c_to, $c_name, $c_link, $c_part, $c_nr, $c_title, $c_color, $c_bcolor, $c_big, $c_canceled;

	$sql = 'SELECT *
			FROM ' . $this->table_prefix . 'calendar_categories
			WHERE category_id = ' . $upcoming['category'];
	$result = $db->sql_query($sql);
	$row = $db->sql_fetchrow($result);
	
	$sql = 'SELECT *
			FROM ' . $this->table_prefix . 'posts
			WHERE post_id = ' . $upcoming['post_id']; 
	$post_result = $db->sql_query($sql);
	$post_row = $db->sql_fetchrow($post_result);
	$db->sql_freeresult($post_result);
	if ($post_row)
	{
		if ($post_row['post_visibility'] == 1)
		{
			$auth_array = $auth->acl_raw_data($user->data['user_id'], 'f_read', $post_row['forum_id']);
			if (isset($auth_array[$user->data['user_id']][$post_row['forum_id']]['f_read']) && $auth_array[$user->data['user_id']][$post_row['forum_id']]['f_read'])
			{	
				$bdate = '';
				$event_date = $upcoming['date_from'];
				if ($upcoming['calendar_repeat'])
				{
					$bdate = 'd=' . strtotime(($upcoming['date_from'])) . '&';
					$upcoming['date_to'] = '0000-00-00';
				}
				$link 	= append_sid($phpbb_root_path . 'viewtopic.php?' . $bdate . 'p=' . $post_row['post_id'] . '#p' . $post_row['post_id']);
				$number = 0;
				$nr = '';
				$part='';

				if($config['calendar_number_participating'])
				{
					$sql = 'SELECT *
							FROM ' . $this->table_prefix . 'calendar_participants
							WHERE post_id = ' . $upcoming['post_id'] . "
								AND event_date = '" . $event_date  . "'";
					$part_result = $db->sql_query($sql);
					while($part_row = $db->sql_fetchrow($part_result))
					{
						$sql = 'SELECT user_colour, username
								FROM ' . USERS_TABLE . '
								WHERE user_id = ' . $part_row['user_id'];
						$user_result = $db->sql_query($sql);
						$user_row = $db->sql_fetchrow($user_result);
	
						if ($part_row['participants'] == 'yes')
						{
							$number += intval($part_row['number']);
							if($config['calendar_participants_name'])
							{
								$part .= chr(10) . $user_row['username'] . ' [' . intval($part_row['number']) . ']';
							}
						}
					}
				}
				$c_c++;
				$c_from[$c_c] = date("Y-m-d", strtotime($upcoming['date_from']));
				$c_to[$c_c] = ($upcoming['date_to'] && $upcoming['date_to'] <> '0000-00-00') ? date("Y-m-d", strtotime($upcoming['date_to'])) : '';
				$c_name[$c_c] = $upcoming['event_name'];
				$c_title[$c_c] = $post_row['post_subject']; 
				$c_part[$c_c] = $part;
				$c_nr[$c_c] = $number;
				$c_link[$c_c] = $link;
				$c_color[$c_c] = $row['color'];
				$c_bcolor[$c_c] = $row['bcolor'];
				$c_big[$c_c] = $row['big'];
				$c_canceled[$c_c] = $upcoming['canceled'];
			}
		}
	}
/*	else
	{
		$sql = 'DELETE FROM ' . CALENDAR_TABLE . ' 
				WHERE post_id = ' . $upcoming['post_id']; 
		$result = $db->sql_query($sql);
	}*/
}
function date_format ($event_date, $time)
{
	global $user, $config;
	
	$d_f	= explode('-',$event_date . '-0-0-0');
	$d = mktime($d_f[3], $d_f[4], $d_f[5], $d_f[1], $d_f[2], $d_f[0]);
	date_default_timezone_set($user->data['user_timezone']);
	$r	= explode('-',date("Y-m-j-H-i-s", $d));
	date_default_timezone_set($config['board_timezone']);
	if($time)
	{
		$date_format = $user->data['user_dateformat'];
	}
	else
	{
		$date_format = trim(preg_replace(array('/a/','/A/','/B/','/g/','/G/','/h/','/H/','/i/','/s/','/u/','/:/'),'',$user->data['user_dateformat']),' ,');
	}
	return $user->create_datetime()
		->setDate($r[0],$r[1],$r[2])
		->setTime($r[3],$r[4],$r[5])
		->format($date_format, true);
}
