<?php
/**
* @package msta calendar Extension
*/

namespace msta\calendar\includes;

class CalendarRenderer extends CalendarBase {
  protected $auth;
  protected $template;
  protected $request;
  protected $phpbb_root_path;
  private $user_datetimeformat = '';
  private $user_dateformat = '';
  private $dateSelectBasicQuery = '';

  public function __construct(\phpbb\db\driver\driver_interface $db, \phpbb\config\config $config, \phpbb\user $user, \phpbb\auth\auth $auth, \phpbb\template\template $template, \phpbb\request\request $request, $phpbb_root_path) {
    parent::__construct($db, $config, $user);
    $this->auth = $auth;
    $this->template = $template;
    $this->request = $request;
    $this->phpbb_root_path = $phpbb_root_path;
    $this->user_datetimeformat = $user->data['user_dateformat'];
    $this->user_dateformat = trim(preg_replace(array('/a/','/A/','/B/','/g/','/G/','/h/','/H/','/i/','/s/','/u/','/:/'), '', $this->user_datetimeformat), ' ,');
    $this->dateSelectBasicQuery =
      'SELECT cal.*, p.forum_id, p.topic_id, p.post_edit_time, p.post_time, p.post_subject, g.group_name AS promoter_name, f.allowed, '.
      'CASE WHEN EXISTS (SELECT 1 FROM '.$this->table_prefix.'calendar_co_promoters WHERE post_id = cal.post_id LIMIT 1) THEN 1 ELSE 0 END AS co_promoters '.
      'FROM '.$this->table_prefix.'calendar AS cal '.
      'INNER JOIN '.$this->table_prefix.'posts AS p ON p.post_id=cal.post_id '.
      'LEFT JOIN '.$this->table_prefix.'groups AS g ON cal.promoter=g.group_id '.
      'LEFT JOIN '.$this->table_prefix.'calendar_forums AS f ON f.forum_id=p.forum_id ';
  }

  public function dateTimeToUserFormatStr ($dt) {
    $dt->setTimezone($this->user_timezone);
    return $dt->format($this->user_dateformat);
  }

  private function getCoPromoters (&$date) {
    if ($date['co_promoters'] == 0) {
      $date['co_promoters'] = null;
      return;
    }
    $sqlResult = $this->db->sql_query('SELECT co.*, g.group_name FROM '.$this->table_prefix.'calendar_co_promoters AS co INNER JOIN '.$this->table_prefix.'groups AS g on co.promoter = g.group_id WHERE co.post_id = '.intval($date['post_id']));
    $date['co_promoters'] = array();
    while ($row = $this->db->sql_fetchrow($sqlResult))
      $date['co_promoters'][] = $row;
    $sqlResult->close();
  }

  private function sqlRowToDate ($row) {
    $result = array();
    foreach ($row as $k => $v) $result[$k] = $v;
    $result['post_list_permission'] = false;
    $result['post_read_permission'] = false;
    $result['from'] = new \DateTime($result['date_from'].' '.$result['time_from'], ($result['time_from'] == '')?$this->user_timezone:$this->utc_timezone);
    $result['to'] = ($result['date_to'] == '')?null:new \DateTime($result['date_to'].' '.$result['time_to'], ($result['time_to'] == '')?$this->user_timezone:$this->utc_timezone);
    $this->getCoPromoters($result);
    return $result;
  }

  public function getDate ($post_id) {
    if ($uid == 0) $flags |= 8;
    if (($from != null && $from->getTimezone() != $this->utc_timezone) || ($to != null && $to->getTimezone() != $this->utc_timezone)) die('Given dates must be in UTC timezone!');
    $row = $this->db->sql_fetchrow($this->db->sql_query($this->dateSelectBasicQuery.'WHERE p.post_id = '.intval($post_id)));
    if ($row) return $this->sqlRowToDate($row);
    else return null;
  }

  /*
   getDates
   $userTZFrom: string of format [Y-m-d H:i], always in user time zone; can be '', if not required
   $userTZTo: string of format [Y-m-d H:i], always in user time zone; can be '', if not required
   $promoters: filter list of promoter grouop ids
   $categories: filter list of category/event ids
   $flags:
    1 = promoters is a blacklist, not whitelist
    2 = categories is a blacklist, not whitelist
    4 = list birthdays
    8 = public events only
  */
  public function getDates ($uid, $from, $to, $promoters, $categories, $flags) {
    if ($uid == 0) $flags |= 8;
    if (($from != null && $from->getTimezone() != $this->utc_timezone) || ($to != null && $to->getTimezone() != $this->utc_timezone)) die('Given dates must be in UTC timezone!');
    // assemble sql query
    $sql = $this->dateSelectBasicQuery.'WHERE p.post_visibility = 1'; // this will get all dates, that have an existing, visible post
    if (($flags & 8) == 8) // performance optimization
      $sql .= " AND cal.public = 1";
    if ($from != null) // exclude dates, that end before range start
      $sql .= " AND (cal.date_to >= '".$from->format('Y-m-d')."' OR cal.date_to = '')";
    if ($to != null) // exclude dates, that begin after range end
      $sql .= " AND cal.date_from <= '".$to->format('Y-m-d')."'";
    if (count($promoters) > 0) // exclude dates, that don't fit to promoters list
      $sql .= ' AND (cal.promoter '.((($flags & 1) == 1)?'NOT ':'').'IN ('.join(', ', $promoters).')'.((($flags & 1) == 0)?' OR cal.post_id IN (SELECT co.post_id FROM '.$this->table_prefix.'calendar_co_promoters AS co WHERE cal.post_id = co.post_id AND co.promoter IN ('.join(', ', $promoters).'))':'').')';
    if (count($categories) > 0) // exclude dates, that don't fit to categories list
      $sql .= ' AND (cal.category '.((($flags & 2) == 2)?'NOT ':'').'IN ('.join(', ', $categories).'))';
    // assemble result array and exclude dates without forum-read-permission
    $sql_result = $this->db->sql_query($sql);
    $result = array();
    if (($flags & 8) == 8) // just convert sql_result to true date array
      foreach ($sql_result as $k => $v)
        $result[] = $this->sqlRowToDate($v);
    else {
      // get read permissions
      $includedForums = array(); // true for all forums, which have the list_topics-permission and therefore will have displayed dates
      $includedForumsRead = array(); // true for all forums, which have the read-permission and will therefore link to the post and not to the event's external url 
      foreach ($sql_result as $k => $v) {
        $includedForums['f'.$v['forum_id']] = false;
        $includedForumsRead['f'.$v['forum_id']] = false;
      }
      foreach ($includedForums as $k => $v) {
        $fid = intval(substr($k, 1));
        $p = $this->auth->acl_raw_data($uid, array('f_read', 'f_list_topics'), $fid); //f_read
        if (isset($p[$uid][$fid]['f_list_topics']) && $p[$uid][$fid]['f_list_topics']) $includedForums[$k] = true;
        if (isset($p[$uid][$fid]['f_read']) && $p[$uid][$fid]['f_read']) $includedForumsRead[$k] = true;
      }
      foreach ($sql_result as $k => $v) // convert sql_result to true array and exclude all dates without read permission
        if ($v['public'] == 1 || $includedForums['f'.$v['forum_id']]) {
          $newDate = $this->sqlRowToDate($v);
          $newDate['post_list_permission'] = $includedForums['f'.$v['forum_id']];
          $newDate['post_read_permission'] = $includedForumsRead['f'.$v['forum_id']];
          $result[] = $newDate;
        }
    }
    $sql_result->close();
    // list birthdays, if required
    if (($flags & 4) == 4) {
      // TODO: Add Birthdays
    }
    return $result;
  }

  /*
   getDayList
   Returns an array of day records, each containing the information on day begin, end, etc.
   $from: The First day's DateTime
   $n: number of days to return
  */
  public function getDayList ($from, $n) {
    $dayBegin = clone $from;
    $dayBegin->setTimezone($this->user_timezone);
    $dayDuration = new \DateInterval('PT30H');
    $list = array();
    for ($i = 0; $i < $n; $i++) {
      $list[$i] = array(
        'dayBegin' => clone $dayBegin,	// begin DateTime of day in utc
        'dayEnd' => null,				// end DateTime of day in utc
        'dayUserTZ' => clone $dayBegin,	// begin DateTime of day in user time zone
        'dayUserTZStr' => '',			// string representation of day in Y-m-d in user time zone
        'dates' => array(),				// list of dates, that occur on this day
      );
      $list[$i]['dayBegin']->setTimezone($this->utc_timezone);
      $list[$i]['dayUserTZ']->setTimezone($this->user_timezone);
      $list[$i]['dayUserTZStr'] = $list[$i]['dayUserTZ']->format('Y-m-d');
      $dayBegin->add($dayDuration);
      $dayBegin->setTime(0, 0, 0);
      $list[$i]['dayEnd'] = clone $dayBegin;
      $list[$i]['dayEnd']->setTimezone($this->utc_timezone);
    }
    return $list;
  }

  /*
   getMonthRepetitionI
   Returns the nearest repetition of given date in a month index. This is an integer, depresenting the date. The least significant 5 bits are the day but, the higher bits the year*12+month. The
   month-bits are continuous, so adding 32 to the index will branch the dext month. But it can contain invalid dates, such as 30th february.
  */
  private function getMonthRepetitionI ($repeatingDate, $currentDateTime, $directionPrevious=false) {
    if ($directionPrevious) die('TODO'); //TODO
    if ($repeatingDate['time_from'] == '') $spl = explode('-', $repeatingDate['date_from'].'-1-1');
    else {
      $df = clone $repeatingDate['from'];
      $df->setTimezone($this->user_timezone);
      $spl = array($df->format('Y'), $df->format('m'), $df->format('d'));
    }
    $repI = intval($spl[0])*32*12 + (intval($spl[1])-1)*32 + intval($spl[2]);
    $currI = intval($currentDateTime->format('Y'))*32*12 + intval($currentDateTime->format('m')-1)*32 + intval($currentDateTime->format('d'));
    if ($currI > $repI)
      $repI += ceil(($currI - $repI + 0.0) / ($repeatingDate['repeat_nr']*32)) * $repeatingDate['repeat_nr']*32;
    return $repI;
  }

  private function lastDayOfMonth ($Y, $m) {
    if ($m == 4 || $m == 6 || $m == 9 || $m == 11) $ld = 30;
    else if ($m == 2) {
      if (($Y % 4) == 0 && (($Y % 100) != 0 || ($Y % 400) == 0)) $ld = 29;
      else $ld = 28;
    } else $ld = 31;
    return $ld;
  }

  /*
   getNearestRepetition
   Returns (usually) the nearest repetition from $currentDateTime. If $directionPrevious is true, it sould be the nearest date before current-, otherwide after currentDateTime. If no further repetition exists,
   an earlier or maybe just the first repetition may be returned. The result is in user timezone and represents the beginning of the repetition.
   Where to care for new repetition rules:
    - repeatInfo
    - main.php:generate_ics // prepare repetition rule
    - edit_date.html
    
  */
  public function getNearestRepetition ($repeatingDate, $currentDateTime, $directionPrevious=false) {
    if ($directionPrevious) die('TODO'); //TODO
    switch (intval($repeatingDate['repeat_mode'])) {
      case 1: // every n month repeating event
        if (intval($repeatingDate['repeat_nr']) <= 0) break;
        $userTZfrom = clone $repeatingDate['from'];
        $userTZfrom->setTimezone($this->user_timezone);
        $repI = $this->getMonthRepetitionI($repeatingDate, $currentDateTime, $directionPrevious);
        for ($i = 0; $i < 100; $i += $repeatingDate['repeat_nr']) { // if after adding 8 years, no valid date is found, it may never exist!
          $Y = (int)floor(($repI >> 5) / 12);
          $m = ($repI >> 5) % 12 + 1;
          $d = $repI & 31;
          if (checkdate($m, $d, $Y)) {
            $result = new \DateTime(str_pad($Y, 4, '0', STR_PAD_LEFT).'-'.str_pad($m, 2, '0', STR_PAD_LEFT).'-'.str_pad($d, 2, '0', STR_PAD_LEFT).' '.$userTZfrom->format('H:i'), $this->user_timezone);
            if ($repeatingDate['to'] !== null && $result > $repeatingDate['to']) break;
            if ($result >= $currentDateTime) return $result;
          }
          $repI += $repeatingDate['repeat_nr']*32;
        }
        break;
      case 2: // every n days repeating event
        if (intval($repeatingDate['repeat_nr']) <= 0) break;
        $result = clone $repeatingDate['from'];
        $result->setTimezone($this->user_timezone);
        $th = intval($result->format('H'));
        $tm = intval($result->format('i'));
        $result->setTime(12, 0); // set to midday, to prevent day missmatch due to summer/winter time gap
        if ($currentDateTime > $repeatingDate['from']) {
          $dayDiff = intval($repeatingDate['from']->diff($currentDateTime, false)->format('%r%a'));
          $result->add(new \DateInterval('P'.(ceil(($dayDiff+0.1) / $repeatingDate['repeat_nr'])*$repeatingDate['repeat_nr']).'D'));
        }
        if ($repeatingDate['to'] !== null && $result > $repeatingDate['to']) break;
        $result->setTime($th, $tm);
        return $result;
      case 3:
        if (intval($repeatingDate['repeat_nr']) == 0) break;
        $userTZfrom = clone $repeatingDate['from'];
        $userTZfrom->setTimezone($this->user_timezone);
        $weekday = intval($userTZfrom->format('w'));
        if (intval($repeatingDate['repeat_nr']) > 0) {
          $ld = 1;
          $weekNr = (int)floor(intval($userTZfrom->format('d')) / 7);
        } else {
          $ld = $this->lastDayOfMonth(intval($userTZfrom->format('Y')), intval($userTZfrom->format('m')));
          $weekNr = (int)floor(($ld - intval($userTZfrom->format('d'))) / 7);
        }
        $repI = $this->getMonthRepetitionI($repeatingDate, $currentDateTime, $directionPrevious);
        for ($i = 0; $i < 100; $i++) {
          $Y = (int)floor(($repI >> 5) / 12);
          $m = ($repI >> 5) % 12 + 1;
          if ($repeatingDate['repeat_nr'] < 0) $ld = $this->lastDayOfMonth($Y, $m);
          $result = new \DateTime(str_pad($Y, 4, '0', STR_PAD_LEFT).'-'.str_pad($m, 2, '0', STR_PAD_LEFT).'-'.str_pad($ld, 2, '0', STR_PAD_LEFT), $this->user_timezone);
          if ($repeatingDate['repeat_nr'] > 0) $result->add(new \DateInterval('P'.((($weekday + 7 - intval($result->format('w'))) % 7) + $weekNr * 7).'D'));
          else $result->sub(new \DateInterval('P'.($weekNr * 7 + ((intval($result->format('w')) - $weekday + 7) % 7)).'D'));
          if ($repeatingDate['to'] !== null && $result > $repeatingDate['to']) break;
          if (($result >= $currentDateTime) && (intval($result->format('m')) == $m)) {
            $th = intval($userTZfrom->format('H'));
            $tm = intval($userTZfrom->format('i'));
            $result->setTime($th, $tm);
            return $result;
          }
          $repI += 32 * abs($repeatingDate['repeat_nr']);
        }
        break;
    }
    $result = clone $repeatingDate['from'];
    $result->setTimezone($this->user_timezone);
    return $result;
  }

  /*
   mapDatesToDays
   Assigns a list of dates to a list of days. Each day will get a dates-field, that is an array holding all dates, that occur on this day. The dates will be directly assigned
   if normal and cloned (but not deeply) if repeating. Repeating dates will get an additional repetitionTime-Field holdung the beginning DateTime of the repetition belonging to the day.
   $dayList must be sorted but may contain gaps.
  */
  public function mapDatesToDays ($dates, &$dayList) {
    if (count($dayList) == 0) return;
    for ($i = 0; $i < count($dates); $i++) {
      if ($dates[$i]['repeat_mode'] == 0) { // non repeating event; may take several days 
        if ($dates[$i]['time_from'] == '') {
          $j = 0;
          while ($j < count($dayList) && $dayList[$j]['dayUserTZStr'] < $dates[$i]['date_from']) $j++; // TODO logarithmic search possible here
          while ($j < count($dayList) && $dayList[$j]['dayUserTZStr'] <= $dates[$i]['date_to']) {
            $dayList[$j]['dates'][] = $dates[$i];
            $j++;
          }
        } else {
          $j = 0;
          while ($j < count($dayList) && $dayList[$j]['dayEnd'] <= $dates[$i]['from']) $j++; // TODO logarithmic search possible here
          while ($j < count($dayList) && $dayList[$j]['dayBegin'] < $dates[$i]['to']) {
            $dayList[$j]['dates'][] = $dates[$i];
            $j++;
          }
        }
      } else {
        if ($dates[$i]['repeat_mode'] == 1 && false) { // this if-block is a performance optimization for monthly repeating events and can be out-commented!
          if ($dates[$i]['repeat_nr'] <= 0) die('repeat month number cannot be zero!');
          $arr = explode('-', $dayList[count($dayList)-1]['dayUserTZStr'].'-1-1');
          $dayListEndI = intval($arr[0])*12*32 + (intval($arr[1])-1)*32 + intval($arr[2]);
          $dateI = $this->getMonthRepetitionI($dates[$i], $dayList[0]['dayUserTZ'], false);
          if ($dates[$i]['date_to'] == '') $dateEndI = PHP_INT_MAX;
          else {
            $arr = explode('-', $dates[$i]['date_to'].'-1-1');
            $dateEndI = intval($arr[0])*12*32 + (intval($arr[1])-1)*32 + intval($arr[2]);
          }
          $j = 0;
          while ($dateI <= $dateEndI) {
            do {
              $arr = explode('-', $dayList[$j]['dayUserTZStr'].'-1-1');
              $dayI = intval($arr[0])*12*32 + (intval($arr[1])-1)*32 + intval($arr[2]);
              if ($dayI >= $dateI) break;
              $j++;
              if ($j >= count($dayList)) break 2;
            } while (true);
            if ($dayI == $dateI) $dayList[$j]['dates'][] = $dates[$i];
            $dateI += $dates[$i]['repeat_nr']*32;
          }
          continue;
        }
        $nextRep = $this->getNearestRepetition($dates[$i], $dayList[0]['dayBegin'], false);
        $j = 0;
        while ($nextRep < $dayList[count($dayList)-1]['dayEnd']) {
          while ($dayList[$j]['dayEnd'] <= $nextRep) {
            $j++;
            if ($j >= count($dayList)) break 2;
          }
          if ($dayList[$j]['dayBegin'] <= $nextRep) {
            $dayList[$j]['dates'][] = array_merge(array(), $dates[$i]);
            $dayList[$j]['dates'][count($dayList[$j]['dates'])-1]['repetitionTime'] = $nextRep;
            $j++;
            if ($j >= count($dayList)) break;
          }
          $nextRep = $this->getNearestRepetition($dates[$i], $dayList[$j]['dayBegin'], false);
          if ($nextRep < $dayList[$j]['dayBegin']) break;
        }
      }
    }
    for ($i = 0; $i < count($dayList); $i++)
      usort($dayList[$i]['dates'], array($this, "compareDates"));
  }

  /*
   dateInfo
   returns a string-representation of the time range covered by the date. This should only be used for mapped dates, that hold the repetitionTime-field, if repeating.
  */
  public function dateInfo ($date) {
    if ($date['repeat_mode'] > 0) {
      if (!$date['repetitionTime']) return 'Repeating date has no repetition time!'; // TODO: Calc repetition time, maybe...
      $event_date_from = clone $date['repetitionTime'];
      $event_date_from->setTimezone($this->user_timezone);
      $spl = explode(':', $date['time_to'].'-00');
      $minDiff = intval($spl[0])*60+intval($spl[1]);
      $spl = explode(':', $date['time_from'].'-00');
      $minDiff = $minDiff - (intval($spl[0])*60+intval($spl[1]));
      $event_date_to = clone $event_date_from;
      $event_date_to->add(new \DateInterval('PT'.$minDiff.'M'));
    } else {
      $event_date_from = clone $date['from'];
      $event_date_from->setTimezone($this->user_timezone);
      $event_date_to = clone $date['to'];
      $event_date_to->setTimezone($this->user_timezone);
    }
    if ($event_date_from->format('Y-m-d') == $event_date_to->format('Y-m-d')) {
      $event_date_info = $event_date_from->format($this->user_dateformat);
      if ($date['time_from'] != '') $event_date_info .= '  '.$event_date_from->format('H:i').' '.$this->user->lang['CALENDAR_TO'].' '.$event_date_to->format('H:i');
      return $event_date_info;
    } else {
      if ($date['time_from'] == '') return $event_date_from->format($this->user_dateformat).' '.$this->user->lang['CALENDAR_TO'].' '.$event_date_to->format($this->user_dateformat);
      else return $event_date_from->format($this->user_datetimeformat).' '.$this->user->lang['CALENDAR_TO'].' '.$event_date_to->format($this->user_datetimeformat);
    }
  }

  /*
   Returns information on the kind of repetition of $date in the current user's language.
  */
  public function repeatInfo ($date) {
    if ($date['repeat_mode'] == 0)
      return '';
    if ($date['repeat_mode'] == 1)
      return $this->user->lang['CALENDAR_REPEAT_MONTH_1'].intval($date['repeat_nr']).$this->user->lang['CALENDAR_REPEAT_MONTH_2'];
    if ($date['repeat_mode'] == 2)
      return $this->user->lang['CALENDAR_REPEAT_DAYS_1'].intval($date['repeat_nr']).$this->user->lang['CALENDAR_REPEAT_DAYS_2'];
    if ($date['repeat_mode'] == 3)
      return $this->user->lang['CALENDAR_REPEAT_WEEK_1'].intval($date['repeat_nr']).$this->user->lang['CALENDAR_REPEAT_WEEK_2'];
    return '';
  }

  /*
   compareDates
   used for sorting
  */
  private function compareDates ($d1, $d2) {
    return strcmp($d1['time_from'], $d2['time_from']);
  }

  /*
   renderDays
   Transfers a given day list with mapped dates to template vars.
  */
  public function renderDays ($dayList, $currentMonth) {
    $today = new \DateTime('now', $this->user_timezone);
    $todayStr = $today->format('Y-m-d');
    for ($i = 0; $i < count($dayList); $i++) {
      $holyday = '';
      $this->template->assign_block_vars('day', array(
        'WEEK'		=> ($this->config['calendar_week_display'] == true)?$dayList[$i]['dayUserTZ']->format("W"):false,
        'INDEX'		=> false,
        'WD_NR'		=> intval($dayList[$i]['dayUserTZ']->format('w')),
        'WD_NAME'	=> $this->user->lang['datetime'][$dayList[$i]['dayUserTZ']->format("l")],
        'DATE'		=> intval($dayList[$i]['dayUserTZ']->format('j')),
        'NODAY'		=> (intval($dayList[$i]['dayUserTZ']->format('m')) == intval($currentMonth))?'':'noday',
        'TODAY'		=> ($dayList[$i]['dayUserTZStr'] == $todayStr)?'today':'',
        'BG'		=> strtolower($dayList[$i]['dayUserTZ']->format('D')),
        'I'			=> $i % 7,
      ));
      for ($j = 0; $j < count($dayList[$i]['dates']); $j++) {
        $date = $dayList[$i]['dates'][$j];
        $this->template->assign_block_vars('day.cdh', array(
          'EVENT_LINK'			=> ($date['post_read_permission'])?append_sid($this->phpbb_root_path.'viewtopic.php?d='.$dayList[$i]['dayBegin']->format('U').'&p='.$date['post_id'].'#p'.$date['post_id']):$date['url'],
          'EVENT_TOPIC'			=> ($date['post_read_permission'])?$date['post_subject']:'',
          'EVENT_DATE'			=> $this->dateInfo($date),
          'EVENT_DESC'			=> $date['description'],
          'EVENT_PROMOTER_ID'	=> $date['promoter'],
          'EVENT_LOC'			=> $date['location'],
          'EVENT_URL'			=> $date['url'],
          'EVENT_PUBLIC'		=> ($date['public'] == 0)?'':'eventPublic',
          'EVENT_TYPE'			=> $this->categoryInfo($date['category'])['name'],
          'EVENT_TITLE'			=> $date['event_name'],
          'EVENT_COLOR' 		=> $this->categoryInfo($date['category'])['color'],
          'EVENT_BCOLOR' 		=> $this->categoryInfo($date['category'])['bcolor'],
          'EVENT_BIG'			=> $this->categoryInfo($date['category'])['big'],
          'EVENT_CANCELED'		=> ($date['canceled'] == 0)?'':'eventCanceled',
        ));
      }
    }
  }

  /*
   renderDayNameList
   Prints the names of the weekdays in user's language to the template's vars.
  */
  public function renderDayNameList () {
    $ws = intval($this->config['calendar_week_start']);
    for ($i = 0; $i < 7; $i++) {
      $d = $i - $ws;
      if ($d < 0) $d += 7;
      if ($d > 6) $d -= 7;
      $day[$d] = $this->user->lang['datetime'][jddayofweek($i - 1, 1)];
    }
    $this->template->assign_vars(array(
      'DAY1' => $day[0],
      'DAY2' => $day[1],
      'DAY3' => $day[2],
      'DAY4' => $day[3],
      'DAY5' => $day[4],
      'DAY6' => $day[5],
      'DAY7' => $day[6],
    ));
  }

  /*
   renderMonthNameList
   Prints the names of the months in user's language to the template's vars.
  */
  public function renderMonthNameList ($currentMonth) {
    $month_name = array(
      1 => $this->user->lang['datetime']['January'],
      2 => $this->user->lang['datetime']['February'],
      3 => $this->user->lang['datetime']['March'],
      4 => $this->user->lang['datetime']['April'],
      5 => $this->user->lang['datetime']['May'],
      6 => $this->user->lang['datetime']['June'],
      7 => $this->user->lang['datetime']['July'],
      8 => $this->user->lang['datetime']['August'],
      9 => $this->user->lang['datetime']['September'],
      10 => $this->user->lang['datetime']['October'],
      11 => $this->user->lang['datetime']['November'],
      12 => $this->user->lang['datetime']['December'],
    );
    for ($i = 1; $i <= 12; $i++)
      $this->template->assign_block_vars('month', array(
        'SELECT' =>'<option'.(($i == $currentMonth)?' selected="selected"':'').' value="'.$i.'">'.$month_name[$i].'</option>',
      ));
  }

  /*
   renderYearList
   Prints some years before and after current date to the template's vars.
  */
  public function renderYearList ($currentYear) {
    for ($i = $currentYear - 2; $i < $currentYear + 8; $i++)
      $this->template->assign_block_vars('year', array(
        'SELECT' =>'<option'.(($i == $currentYear)?' selected="selected"':'').' value="'.$i.'">'.$i.'</option>',
      ));	
  }

  /*
   renderDate
   returns the date information to be used in the viewtopic-post calendar entry to be added to php event's post variables.
   $date as got through getDate(s)
   $currentDate base to get next repetition
  */
  public function renderDate ($date, $currentDate, $showParticipants = true, $showAvaillablePromoters = true, $forUser = 0) {
    // compose date info
    $adminOrGlobalMod = $this->auth->acl_get('a_') == 1 || $this->auth->acl_get('m_') == 1; // check, if the current user is admin or global moderator
    $appLink = append_sid('app.php/calendar/?');
    if ($forUser == 0) $forUser = intval($this->user->data['user_id']);
    $result = array(
      'FROM_TO'					=> '',
      'URL'						=> $date['url'],
      'NAME'					=> $date['event_name'],
      'LINK'					=> $appLink.'calendar/?month='.$date['from']->format('m').'&amp;year='.$date['from']->format('Y'),
      'REPEAT'					=> $this->repeatInfo($date),
      'PROMOTER'				=> ($date['promoter'] == 0)?'':htmlspecialchars($date['promoter_name']),
      'PROMOTER_ID'				=> $date['promoter'],
      'LOCATION'				=> $date['location'],
      'DESC'					=> $date['description'],
      'CANCELED'				=> ($date['canceled'] == 1)?'yes':'',
      'PUBLIC'					=> ($date['public'] == 1)?'yes':'',
      'CATEGORY'				=> $this->categoryInfo($date['category'])['name'],
      'COLOR'					=> $this->categoryInfo($date['category'])['color'],
      'BCOLOR'					=> $this->categoryInfo($date['category'])['bcolor'],
      'BIG'						=> $this->categoryInfo($date['category'])['big'],
      'PARTICIPANTS_LIST'		=> $date['participants'],
      'EVENT_DATE'				=> $date['from']->format('U'),
      'U_ACTION'				=> append_sid($this->phpbb_root_path.'viewtopic.php?f='.$date['forum_id'].'&amp;p='.$date['post_id'].'#p'.$date['post_id']),
      'USER'					=> $this->user->data['username'],
      'APP_LINK'				=> $appLink,
      'POST_ID'					=> $date['post_id'],
      'REPETITIONS'				=> '',
      'CO_PROMOTERS'			=> array(),
      'AVAILLABLE_PROMOTERS'	=> ($showAvaillablePromoters)?$this->promotersAsOptions(!$adminOrGlobalMod, !$adminOrGlobalMod):null,
      'USER_IS_INVITED'			=> false,
      'USER_IS_REGISTERED'		=> false,
    );

    // create list of co-promoters
    if ($date['co_promoters'])
      for ($i = 0; $i < count($date['co_promoters']); $i++)
        $result['CO_PROMOTERS'][] = array(
          'PROMOTER_NAME'		=> htmlspecialchars($date['co_promoters'][$i]['group_name']),
          'PROMOTER_ID'			=> $date['co_promoters'][$i]['promoter'],
          'PROMOTER_COMMENT'	=> $date['co_promoters'][$i]['comment'],
        );

    // create list of availlable repetitions
    $selectedRep = null;
    $n = 0;
    if ($date['repeat_mode'] != 0) {
      $rep = $this->getNearestRepetition($date, $date['from']);
      while (($date['to'] === null || $rep < $date['to']) and ($n < 20)) {
        $n++;
        $repPlus = new \DateTime($rep->format('d-m-Y').' 23:59:01'); // set time lo last minute of day to make getNearestRepetition find the next repetition after current
        $repN = $this->getNearestRepetition($date, $repPlus);
        if ($n == 10 && $repN < $currentDate) $repN = $this->getNearestRepetition($date, $currentDate); 
        $repU = $rep->format('U');
        if ($selectedRep == null && ($repN <= $rep || ($date['to'] !== null && $repN > $date['to']) || $rep >= $currentDate)) $selectedRep = $rep;
        $result['REPETITIONS'] .= '<option value="'.$repU.'" '.(($selectedRep == $rep)?'selected':'').'>'.$rep->format("Y-m-d").'</option>';
        if ($repN <= $rep) break;
        $rep = $repN;
      }
    }
    $date['repetitionTime'] = $selectedRep;
    $result['FROM_TO'] = $this->dateInfo($date);
    if ($selectedRep) {
      $result['EVENT_DATE'] = $selectedRep->format('U');
      $result['LINK'] = $appLink.'calendar/?month='.$selectedRep->format('m').'&amp;year='.$selectedRep->format('Y');
    }

    if (!$showParticipants) return $result;
    // create list of participants of selected repetition
    if ($date['participants'] > 0) {
      $result['PARTICIPANTS'] = array();
      $numbers = array(0, 0, 0);
      $setUsers = array();
      $sqlParticipants = $this->db->sql_query('SELECT part.participate, part.number, part.comments, part.date, u.username, part.user_id, u.user_colour FROM '.$this->table_prefix.'calendar_participants AS part INNER JOIN '.$this->table_prefix.'users AS u ON part.user_id > 0 AND part.user_id = u.user_id WHERE (u.user_type = '.USER_NORMAL.' OR u.user_type = '.USER_FOUNDER.') AND part.post_id = '.$date['post_id'].(($selectedRep)?" AND (part.event_date = '".$selectedRep->format("Y-m-d")."' OR part.participate = 4)":"")." ORDER BY part.participate, part.date");
      while ($row = $this->db->sql_fetchrow($sqlParticipants)) {
        switch ($row['participate']) {
          case 0:
            if (intval($row['user_id']) == $forUser) $result['USER_IS_REGISTERED'] = true;
            $numbers[2]++;
            $strPart = $this->user->lang['CALENDAR_NO'];
            break;
          case 1:
            if (intval($row['user_id']) == $forUser) $result['USER_IS_REGISTERED'] = true;
            $numbers[1]++;
            $strPart = $this->user->lang['CALENDAR_MB'];
            break;
          case 2:
            if (intval($row['user_id']) == $forUser) $result['USER_IS_REGISTERED'] = true;
            $numbers[0]++;
            $strPart = $this->user->lang['CALENDAR_YES'];
            break;
          case 3:
            if (intval($row['user_id']) == $forUser) $result['USER_IS_REGISTERED'] = true;
            $numbers[0]++;
            $strPart = $this->user->lang['CALENDAR_REPLACE'];
            break;
          case 4:
            if (intval($row['user_id']) == $forUser) $result['USER_IS_INVITED'] = true;
            if ($setUsers['U'.intval($row['user_id'])]) continue 2;
            $strPart = $this->user->lang['CALENDAR_INVITED'];
            break;
        }
        $setUsers['U'.intval($row['user_id'])] = true;
        $result['PARTICIPANTS'][] = array(
          'PARTICIPANTS_USER'		=> $row['username'],
          'PARTICIPANTS_USER_LINK'	=> append_sid($this->phpbb_root_path.'memberlist.php?mode=viewprofile&amp;u='.intval($row['user_id'])),
          'PARTICIPANTS_COLOUR'		=> $row['user_colour'],
          'PARTICIPANTS_NUMBER'		=> $row['number'],
          'PARTICIPANTS_PART'		=> $strPart,
          'PARTICIPANTS_COMMENTS'	=> $row['comments'],
          'PARTICIPANTS_DATE'		=> $this->dateTimeToUserFormatStr(\DateTime::createFromFormat('Y-m-d-H-i', $row['date'])),
        );
      }
      $sqlParticipants->close();
      $groups = array();
      $sqlParticipants = $this->db->sql_query('SELECT part.participate, part.number, part.comments, part.date, g.group_name, g.group_id, g.group_colour FROM '.$this->table_prefix.'calendar_participants AS part INNER JOIN '.$this->table_prefix.'groups AS g ON part.user_id < 0 AND part.user_id = -g.group_id WHERE part.post_id = '.$date['post_id']." AND part.participate = 4 ORDER BY part.participate, part.date");
      while ($row = $this->db->sql_fetchrow($sqlParticipants)) {
        $groups[] = intval($row['group_id']);
        $result['PARTICIPANTS'][] = array(
          'PARTICIPANTS_USER'		=> '*'.$row['group_name'],
          'PARTICIPANTS_USER_LINK'	=> append_sid($this->phpbb_root_path.'memberlist.php?mode=group&amp;g='.intval($row['group_id'])),
          'PARTICIPANTS_COLOUR'		=> $row['group_colour'],
          'PARTICIPANTS_NUMBER'		=> $row['number'],
          'PARTICIPANTS_PART'		=> $this->user->lang['CALENDAR_INVITED'],
          'PARTICIPANTS_COMMENTS'	=> $row['comments'],
          'PARTICIPANTS_DATE'		=> $this->dateTimeToUserFormatStr(\DateTime::createFromFormat('Y-m-d-H-i', $row['date'])),
        );
      }
      $sqlParticipants->close();
      if (count($groups) > 0) {
        $row = $this->db->sql_fetchrow($this->db->sql_query('SELECT user_id FROM '.$this->table_prefix.'user_group WHERE user_id = '.$forUser.' AND user_pending = 0 AND group_id IN ('.join(', ', $groups).') LIMIT 1'));
        if ($row) $result['USER_IS_INVITED'] = true;
      }
      $result['PARTICIPANTS_COUNT'] = $numbers[0].'&nbsp;/&nbsp;'.$numbers[1].'&nbsp;/&nbsp;'.$numbers[2];
    }
    return $result;
  }



  private function utc_to_ics ($t) {
    $date = new \DateTime();
    $date->setTimestamp($t);
    return $date->format('Ymd\THis\Z');
  }

  private function date_to_ics ($d, $t) {
    if ($t == '') return ';VALUE=DATE:'.join('', explode('-', $d));
    else return ':'.join('', explode('-', $d)).'T'.join('', explode(':', $t)).'00Z';
  }

  private function encode_ics_str ($str) {
    if (strpos($str, '&#') !== false) { // convert entities for chars with more than two bytes, which don't fit into mysql-db
      $str = explode('&#', $str);
      for ($i = 1; $i < count($str); $i++) {
        $j = strpos($str[$i], ';');
        if ($j === false) {
          $str[$i] = '&#'.$str[$i];
          continue;
        }
        $c = intval(substr($str[$i], 0, $j));
        if ($c <= 65535)
          $str[$i] = '&#'.$str[$i];
        else
          $str[$i] = html_entity_decode('&#'.$str[$i]);
      }
      $str = join('', $str);
    }
    return str_replace($this->icsSpecialChars, $this->icsSpecialCharsEscaped, $str);
  }

  /*
   datesToICS
   Converts an array of dates to an ics string-representation.
   $dates: the array of dates
   $uid: the user id of the user who asks fpr the dates (not always the id of loggen-in user)
   $promoters: array of promoter ids
   $promBl: true, if the promoter list is a blacklist, false for whitelist
   $serverName: domain of the server - required to generate the UIDs
   $descExt: string where each character can represent a flag to include additional information in the date's DESCRIPTION-field:
    u - adds the event's URL
  */
  public function datesToICS ($dates, $uid, $promoters, $promBl, $serverName, $descExt='') {
    $ics = "BEGIN:VCALENDAR\n"
     ."PRODID:-//phpBB/msta/calendar//\n"
     ."VERSION:2.0\n"
     ."METHOD:PUBLISH\n";
    for ($i = 0; $i < count($dates); $i++) {
      // prepare repetition rule
      switch (intval($dates[$i]['repeat_mode'])) {
        case 0:
          $repeatitionRule = '';
          break;
        case 1:
          if ($dates[$i]['repeat_nr'] <= 0) continue 2;
          $repeatitionRule = 'RRULE:FREQ=MONTHLY;';
          if ($dates[$i]['date_to'] != '') $repeatitionRule .= 'UNTIL='.$dates[$i]['to']->format('Ymd').';';
          $repeatitionRule .= 'INTERVAL='.intval($dates[$i]['repeat_nr'])."\n";
          break;
        case 2:
          if ($dates[$i]['repeat_nr'] <= 0) continue 2;
          $repeatitionRule = 'RRULE:FREQ=DAILY;';
          if ($dates[$i]['date_to'] != '') $repeatitionRule .= 'UNTIL='.$dates[$i]['to']->format('Ymd').';';
          $repeatitionRule .= 'INTERVAL='.intval($dates[$i]['repeat_nr'])."\n";
          break;
        case 3:
          if ($dates[$i]['repeat_nr'] == 0) continue 2;
          $repeatitionRule = 'RRULE:FREQ=MONTHLY;';
          if ($dates[$i]['date_to'] != '') $repeatitionRule .= 'UNTIL='.$dates[$i]['to']->format('Ymd').';';
          $repeatitionRule .= 'INTERVAL='.abs(intval($dates[$i]['repeat_nr'])).';';
          $userTZfrom = clone $dates[$i]['from'];
          $userTZfrom->setTimezone($this->user_timezone);
          $weekdayName = ucfirst(substr($userTZfrom->format('D'), 0, 2));
          if ($dates[$i]['repeat_nr'] > 0)
            $weekNr = 1 + (int)floor(intval($userTZfrom->format('d')) / 7);
          else
            $weekNr = -1 - (int)floor(($this->lastDayOfMonth(intval($userTZfrom->format('Y')), intval($userTZfrom->format('m'))) - intval($userTZfrom->format('d'))) / 7);
          $repeatitionRule .= 'BYDAY='.$weekNr.$weekdayName."\n";
          break;
        default:
          continue 2;
      }
      // compose description
      $icsDesc = array($dates[$i]['description']);
      if ($dates[$i]['co_promoters'] && ((count($promoters) > 0 && !$promBl) || ($uid != 0))) { // check if search for co-promoters is necessary - add co-promoter-comments, if user is member of co-promoter-group or if co-promoter is explicitly in promoter filter
        if ($uid != 0)
          $sqlGroupMemberships = $this->db->sql_query('SELECT group_id FROM '.$this->table_prefix.'user_group WHERE user_pending = 0 AND user_id = '.intval($uid));
        for ($j = 0; $j < count($dates[$i]['co_promoters']); $j++)
          if (strval($dates[$i]['co_promoters'][$j]['comment']) != '' && in_array(intval($dates[$i]['co_promoters'][$j]['promoter']), $promoters))
            $icsDesc[] = strval($dates[$i]['co_promoters'][$j]['comment']);
          else if ($uid != 0)
            foreach ($sqlGroupMemberships as $mem)
              if ($dates[$i]['co_promoters'][$j]['promoter'] == $mem['group_id']) {
                $icsDesc[] = strval($dates[$i]['co_promoters'][$j]['comment']);
                break;
              }
      }
      $url = ($vUser == 'public' || !$dates[$i]['post_read_permission'])?htmlspecialchars_decode($dates[$i]['url']):generate_board_url().'/viewtopic.php?p='.$dates[$i]['post_id'].'#p'.$dates[$i]['post_id'];
      $descriptionExtend = '';
      $descriptionExtendQ = str_split($descExt);
      for ($j = 0; $j < count($descriptionExtendQ); $j++)
        if ($descriptionExtendQ[$j] == 'u' && $url != '')
          $descriptionExtend .= "\nURL: ".$url;
      if ($descriptionExtend != '')
        $icsDesc[] = $descriptionExtend;
      // compose categories
      $icsCategories = array($this->categoryInfo($dates[$i]['category'])['name']);
      $icsCategories = array_merge($icsCategories, explode(',', $this->promoterInfo($dates[$i]['promoter'])['calendar_category']));
      $icsCategories = array_merge($icsCategories, explode(',', $this->categoryInfo($dates[$i]['category'])['ics_additional']));
      for ($j = 0; $j < count($icsCategories); $j++)
        $icsCategories[$j] = $this->encode_ics_str(htmlspecialchars_decode($icsCategories[$j]));
      // add date to ics
      $ics .= ""
       ."BEGIN:VEVENT\n"
       ."DTSTAMP:".$this->utc_to_ics($dates[$i]['post_time'])."\n"
       ."LAST-MODIFIED:".(($dates[$i]['post_edit_time'] > $dates[$i]['post_time'])?$this->utc_to_ics($dates[$i]['post_edit_time']):$this->utc_to_ics($dates[$i]['post_time']))."\n"
       ."UID:".$dates[$i]['post_id']."@".$serverName."\n"
       ."SUMMARY:".$this->encode_ics_str(((intval($dates[$i]['canceled']) == 1)?'['.$this->user->lang['CALENDAR_CANCELED_SHORT'].'] ':"").htmlspecialchars_decode($dates[$i]['event_name']))."\n"
       ."LOCATION:".$this->encode_ics_str(htmlspecialchars_decode($dates[$i]['location']))."\n"
       ."URL:".$this->encode_ics_str($url)."\n"
       ."DESCRIPTION:".$this->encode_ics_str(htmlspecialchars_decode(join("\n----------------\n", $icsDesc)))."\n"
       ."CATEGORIES:".join(',', $icsCategories)."\n"
       ."CLASS:".(($dates[$i]['public'] == 1)?"PUBLIC":"PRIVATE")."\n"
       ."DTSTART".$this->date_to_ics($dates[$i]['date_from'], $dates[$i]['time_from'])."\n"
       ."DTEND".$this->date_to_ics(((intval($dates[$i]['repeat_mode']) == 0)?$dates[$i]['date_to']:$dates[$i]['date_from']), $dates[$i]['time_to'])."\n"
       .$repeatitionRule
       .((intval($dates[$i]['canceled']) == 1)?"STATUS:CANCELLED\n":"")
       ."END:VEVENT\n";
    }
    $ics .= "END:VCALENDAR";
    return $ics;
  }

}
