(function () {
  var info = new Array();
  var hint = null;
  var promoters = null;
  var promotersLoading = false;

  function createHint (i) {
    hint = new Object();
    hint.box = document.createElement('div');
	hint.pin = document.createElement('div');
    hint.pin.className = 'pin';
    hint.box.className = 'hoverhint';
    hint.box.appendChild(hint.pin);
    hint.title = document.createElement('p');
    hint.title.className = 'title';
    hint.box.appendChild(hint.title);
    hint.topic = document.createElement('span');
    var p = document.createElement('p');
    p.className = 'info';
    hint.eventPromoter = document.createElement('span');
    p.appendChild(hint.eventPromoter);
    hint.eventType = document.createElement('span');
    p.appendChild(hint.eventType);
    hint.eventPublic = document.createElement('span');
    hint.eventPublic.className = 'hintPublic';
    hint.eventPublic.textContent = '{L_CALENDAR_PUBLIC_SHORT}';
    p.appendChild(hint.eventPublic);
    hint.eventCanceled = document.createElement('span');
    hint.eventCanceled.className = 'hintCanceled';
    hint.eventCanceled.textContent = '{L_CALENDAR_CANCELED_SHORT}';
    p.appendChild(hint.eventCanceled);
    hint.topicText = document.createElement('span');
    hint.topicText.appendChild(document.createTextNode(', {L_CALENDAR_TOPIC_TITLE}'));
    hint.topicText.appendChild(hint.topic);
    p.appendChild(hint.topicText);
    hint.topic.className = 'topic';
    hint.box.appendChild(p);
    p = document.createElement('p');
    p.appendChild(document.createTextNode('{L_CALENDAR_FROM} '));
    hint.date = document.createElement('span');
    p.appendChild(hint.date);
    hint.box.appendChild(p);
    p = document.createElement('p');
    p.appendChild(document.createTextNode('{L_CALENDAR_EVENT_LOCATION}: '));
    hint.location = document.createElement('span');
    p.appendChild(hint.location);
    p.className = 'location';
    hint.box.appendChild(p);
    hint.description = document.createElement('p');
    hint.description.className = 'description';
    hint.box.appendChild(hint.description);
  }

  function showPromoter () { // The names of the promoter groups are not included into the event info as this is a compute-intensive task. They are fetched when the are required.
    if (promotersLoading) return;
    function loaded () {
      if (promoters['G'+hint.eventPromoter.promoterId])
        hint.eventPromoter.textContent = promoters['G'+hint.eventPromoter.promoterId]+': ';
    }
    if (promoters) loaded();
    else {
      promotersLoading = true;
      var xhttp = new XMLHttpRequest();
      xhttp.onreadystatechange = function () {
        if (this.readyState != 4 || this.status != 200) return;
        promotersLoading = false;
        var info = null;
        try {
          info = JSON.parse(xhttp.responseText);
        } catch (e) {
          info = { promoters: [] };
        }
        promoters = new Array();
        for (var i = 0; i < info.promoters.length; i++)
          promoters['G'+info.promoters[i].id] = info.promoters[i].title;
        loaded();
      };
      var url = document.getElementById('calendarUrl').getAttribute('url')+'?action=filterinfo';
      xhttp.open("GET", url, true);
      xhttp.send();
    }
  }

  function showHint (ev) {
    if (!hint) createHint();
    if (!hint.box.parentElement) document.body.appendChild(hint.box);
    this.style.cursor = 'none';
    moveHint(ev);
    var i = parseInt(this.getAttribute('hintIndex'));
    hint.title.textContent = info[i].eventTitle;
    hint.topic.textContent = info[i].eventTopic;
    hint.eventPromoter.textContent = '';
    hint.eventPromoter.promoterId = info[i].eventPromoterId;
    showPromoter();
    hint.topicText.style.display = (hint.title.textContent.indexOf(info[i].eventTopic) == 0)?'none':'inline';
    hint.date.textContent = info[i].eventDate;
    hint.location.textContent = info[i].eventLocation;
    hint.location.parentElement.style.display = (info[i].eventLocation == '')?'none':'block';
    hint.description.textContent = info[i].eventDesc;
    hint.description.style.display = (info[i].eventDesc == '')?'none':'block';
    hint.eventPublic.style.display = (info[i].eventPublic == '1')?'inline':'none';
    hint.eventCanceled.style.display = (info[i].eventCanceled)?'inline':'none';
    hint.eventType.textContent = info[i].eventType;
  }

  function moveHint (ev) {
    if (!hint || !hint.box.parentElement) return;
    hint.box.style.top = ev.clientY + hint.pin.offsetHeight + 'px';
    hint.pin.style.left = '50%';
    var x = ev.clientX - hint.pin.offsetLeft - hint.pin.offsetWidth/2;
    if (x < 0) {
      x = 0;
      hint.pin.style.left = ev.clientX - hint.pin.offsetWidth/2 + 'px';
    } else if (x + hint.box.offsetWidth > document.documentElement.clientWidth) {
      x = document.documentElement.clientWidth - hint.box.offsetWidth;
      hint.pin.style.left = ev.clientX - x - hint.pin.offsetWidth/2 + 'px'
    }
    hint.box.style.left = x + 'px';
  }

  function hideHint (ev) {
    if (!hint) return;
    if (hint.box.parentElement) hint.box.parentElement.removeChild(hint.box);
  }

  // load table
  var cal = document.getElementById('calendar');
  if (!cal) cal = document.getElementById('index-calendar');
  if (!cal) return;
  var events = cal.getElementsByTagName('a');
  for (var i = 0; i < events.length; i++) {
    var className = events[i].className.split(' ');
    if (className.includes('event')) {
      var ni = new Object();
      for (var j = 0; j < events[i].childNodes.length; j++)
        if (events[i].childNodes[j].nodeType == 1 && events[i].childNodes[j].className != '')
          ni[events[i].childNodes[j].className.split(' ')[0]] = events[i].childNodes[j].textContent;
      for (var j = 2; j < className.length; j++)
        ni[className[j]] = true;
      info.push(ni);
      ni = null;
      events[i].addEventListener('mouseover', showHint, false);
      events[i].addEventListener('mousemove', moveHint, false);
      events[i].addEventListener('mouseout', hideHint, false);
      events[i].setAttribute('hintIndex', info.length-1);
    }
  }
  events = null;
})();
